/**
 * \file mgui/customdraw.h
 * \brief Definitions for MGUI::CUSTOMDRAW.
 *
 * \if NODOC
 * $Id: customdraw.h_v 1.4 2011/11/22 13:59:09 mju Exp $
 *
 * $Log: customdraw.h_v $
 * Revision 1.4  2011/11/22 13:59:09  mju
 * Fix dox.
 *
 * Revision 1.3  2005/11/10 18:45:34  mju
 * Add setRect.
 *
 * Revision 1.2  2004/05/12 17:43:53  mju
 * Change setbase to not have drawing stage.
 * Add setdrawstage.
 *
 * Revision 1.1  2004/05/03 20:39:55  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_CUSTOMDRAW_H
#define	INC_MGUI_CUSTOMDRAW_H

#ifndef	INC_MI32_RECT_H
#include <mi32/rect.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
namespace MGD {
	class CONTEXT;
	}
#endif


namespace MGUI {

//! Base class for custom draw notification supported by selected controls.
//!
//! Custom draw allows an application to easily change selected drawing aspects or manually draw
//! selected items without having to do a full owner draw.
//!
//! At the beginning of each 'paint' cycle, the control's v_OnCustomDraw function will be called with
//! a CUSTOMDRAW-derived class instance having its drawing stage set to DRAWSTAGE_PrePaint.  The value
//! the application returns from this first notification dictates how and when the control sends
//! subsequent custom draw notifications fo rthe rest of that paint cycle.  The application can return
//! a combination of the following flags in response to this first notification.
//! 
//! 	RETVALUE_DoDefault			Control will draw itself, no additional notifications will be sent for this paint cycle.
//! 	RETVALUE_NotifyItemDraw		v_OnCustomDraw will be called for any item-specific drawing, before and after items are drawn.
//! 	RETVALUE_NotifyPostPaint	v_OnCustomDraw will be called when painting cycle for entire control is complete
//! 	RETVALUE_SkipDefault			Control will not perform any painting at all
//!
//! If the application returns RETVALUE_NotifyItemDraw for the initial prepaint notification, notifications will be sent
//! for each items it draws during that paint cycle.  These item-specific notifications will have a drawing stage of
//! DRAWSTAGE_ItemPrePaint.  Notification for when it it finished drawing the item may then be requested by returning
//! RETVALUE_NotifyPostPaint.  Otherwise return RETVALUE_DoDefault and no post-paint notification will be sent.  If the
//! application draws the entire item, return RETVALUE_SkipDefault and the control will not draw any portion of the item.
class CUSTOMDRAW {
	public:

		//! Drawing stage.
		//  Values chosen to match Windows common controls.
		enum DRAWSTAGE {
			DRAWSTAGE_PrePaint =						0x00000001,
			DRAWSTAGE_PostPaint =					0x00000002,
			DRAWSTAGE_PreErase =						0x00000003,
			DRAWSTAGE_PostErase =					0x00000004,
			DRAWSTAGE_CtrlPrePaint =				DRAWSTAGE_PrePaint,
			DRAWSTAGE_CtrlPostPaint =				DRAWSTAGE_PostPaint,
			DRAWSTAGE_CtrlPreErase =				DRAWSTAGE_PreErase,
			DRAWSTAGE_CtrlPostErase =				DRAWSTAGE_PostErase,
			DRAWSTAGE_Item =							0x00010000,
			DRAWSTAGE_ItemPrePaint =				DRAWSTAGE_Item | DRAWSTAGE_PrePaint,
			DRAWSTAGE_ItemPostPaint =				DRAWSTAGE_Item | DRAWSTAGE_PostPaint,
			DRAWSTAGE_ItemPreErase =				DRAWSTAGE_Item | DRAWSTAGE_PreErase,
			DRAWSTAGE_ItemPostErase =				DRAWSTAGE_Item | DRAWSTAGE_PostErase,
			DRAWSTAGE_SubItem =						0x00020000,
			DRAWSTAGE_SubItemPrePaint =			DRAWSTAGE_SubItem | DRAWSTAGE_ItemPrePaint,
			DRAWSTAGE_SubItemPostPaint =			DRAWSTAGE_SubItem | DRAWSTAGE_ItemPostPaint,
			};

		//! Item state
		//  Values chosen to match Windows common controls.
		enum ITEMSTATE {
			ITEMSTATE_Normal =				0x0000,
			ITEMSTATE_Selected =				0x0001,
			ITEMSTATE_Grayed =				0x0002,
			ITEMSTATE_Disabled =				0x0004,
			ITEMSTATE_Checked =				0x0008,
			ITEMSTATE_Focus =					0x0010,
			ITEMSTATE_Default =				0x0020,
			ITEMSTATE_Hot =					0x0040,
			ITEMSTATE_Marked =				0x0080,
			ITEMSTATE_Indeterminant =		0x0100,
			};

		//! Values to return from control-specific v_OnCustomDraw override.
		//! The values the application can return depend on the current drawing stage.
		enum RETVALUE {
			RETVALUE_DoDefault =				0x00,	//!< Let control do all drawing, valid for DRAWSTAGE_...PrePaint
			RETVALUE_SkipDefault =			0x04,	//!< Indicate item was completely drawn manually, valid for DRAWSTAGE_...PrePaint
			RETVALUE_NotifyPostPaint =		0x10,	//!< Request notification after painting item, valid for DRAWSTAGE_...PrePaint
			RETVALUE_NotifyItemDraw =		0x20,	//!< Request notification before/after drawing each item, valid for DRAWSTAGE_CtrlPrePaint
			RETVALUE_NotifySubItemDraw =	0x20,	//!< Request notification before/after drawing each subitem, valid for DRAWSTAGE_CtrlPrePaint
			RETVALUE_NotifyPostErase =		0x40,	//!< Request notification after erasing item, valid for DRAWSTAGE_CtrlPrePaint
			};

		MGD::CONTEXT* GetDrawingContext (
			) const { return (m_pDC); }

		DRAWSTAGE GetDrawStage (
			) const { return (m_drawstage); }

		ID GetID (
			) const { return (m_id); }

		ITEMSTATE GetItemState (
			) const { return (m_itemstate); }

		const LRECT2D& GetRect (
			) const { return (m_rect); }

		void SetDrawStage (
			DRAWSTAGE drawstage
			) { m_drawstage = drawstage; }

	protected:

		CUSTOMDRAW (
			);

		CUSTOMDRAW (
			DRAWSTAGE drawstage,
			ITEMSTATE itemstate,
			INT32 ItemNumber,
			ID id,
			const LRECT2D& rect
			);

		~CUSTOMDRAW ()
			{ }

		INT32 GetItemNumber (
			) const { return (m_ItemNumber); }

		void SetBase (
			ITEMSTATE itemstate,
			INT32 ItemNumber,
			ID id,
			const LRECT2D& rect
			);

		void SetDrawingContext (
			MGD::CONTEXT *pDC
			) { m_pDC = pDC; }

		void SetRect (
			INT32 xinit,
			INT32 yinit,
			INT32 xlast,
			INT32 ylast
			) { m_rect.Set(xinit,yinit,xlast,ylast); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		DRAWSTAGE m_drawstage;
		ITEMSTATE m_itemstate;
		INT32 m_ItemNumber;
		ID m_id;
		LRECT2D m_rect;
		MGD::CONTEXT *m_pDC;
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(CUSTOMDRAW::ITEMSTATE)
DEFINE_ENUM_OP_BITWISE(CUSTOMDRAW::RETVALUE)
#endif

}	// End namespace MGUI

#endif	// INC_MGUI_CUSTOMDRAW_H
