/**
 * \file mgui/grid.h
 * \brief MGUI::CTRL_GRID class definitions
 *
 * \if NODOC
 * $Id: grid.h_v 1.83 2011/11/09 16:04:22 mju Exp $
 *
 * $Log: grid.h_v $
 * Revision 1.83  2011/11/09 16:04:22  mju
 * Remove mfc defns.
 *
 * Revision 1.82  2011/09/01 19:41:15  mju
 * No legacy macro.
 *
 * Revision 1.81  2010/05/18 15:05:52  mju
 * Add editFinish.
 *
 * Revision 1.80  2010/05/14 14:09:07  mju
 * Make getItemText const.
 *
 * Revision 1.79  2009/08/20 20:09:14  mju
 * Add setDelegateOnSize.
 *
 * Revision 1.78  2009/06/22 15:48:26  mju
 * Make getColumnOrderArray const.
 *
 * Revision 1.77  2008/10/08 20:49:49  bcolombe
 * Added GetItemText
 *
 * Revision 1.76  2008/10/08 20:38:51  bcolombe
 * Added deleteitem that allows to not immediately redraw grid
 *
 * Revision 1.71  2008/09/17 20:48:43  mju
 * Add style_NoOuterFrame.
 *
 * Revision 1.69  2008/08/29 15:40:47  mju
 * Add style_SizeInPixels.
 *
 * Revision 1.67  2007/09/05 21:06:32  mju
 * Add redrawDisable/Enable.
 *
 * Revision 1.65  2007/07/23 13:24:35  mju
 * Add style_MultiSelect.
 *
 * Revision 1.64  2007/02/13 18:10:21  dwilliss
 * Added virtual method for tooltip to template version
 *
 * Revision 1.60  2007/02/12 21:49:59  dwilliss
 * Added a SetRowHeight method
 *
 * Revision 1.54  2005/08/11 22:55:14  scowan
 * Fixed multiple issues with using icons in owner drawn data.
 *
 * Revision 1.52  2005/05/23 23:12:16  dfriberg
 * Added virtual methods for column resize and order change
 *
 * Revision 1.31  2003/01/06 17:47:59  dwilliss
 * Added members to keep track of previously selected item so that we can
 * implement multiselect
 *
 * Revision 1.1  2002/02/04 18:48:22  dwilliss
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_GRID_H
#define	INC_MGUI_GRID_H

#ifndef	INC_MGUI_CTRL_H
	#include <mgui/ctrl.h>
#endif

#ifndef	INC_MI32_MISTRING_H
	#include <mi32/mistring.h>
#endif

#ifndef	INC_MI32_COLOR_H
	#include <mi32/color.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
struct LPOINT2D;
namespace MGD {
	class CONTEXT;
	}
namespace MGUI {
	class IMAGELIST;
	class TOOLTIP;
	}
#endif


namespace MGUI {

//------------------------------------------------------------------------------------------------
//! Grid user interface class.
//! There are two general ways to work with lists, "standard" and "owner-data" (virtual).  In the
//! standard configuration the list control itself contains all of the data (strings, icons, states)
//! for each list entry.  In the "owner-data" configuration, the data is stored outside the list
//! and the OnGetDispInfo and OnOwnerDataCacheHint methods are overridden (or set when using the
//! template) and will be called by the list control when data is needed to be displayed in the list.
class CTRL_GRID : public MGUI::CTRL {
	public:

		enum CELLTYPE {
			CELLTYPE_Text = 0,					// Simple text cell (default)
			CELLTYPE_Button = 1,					// Simple pushbutton
			CELLTYPE_DropDown = 2,				// Dropdown list
			CELLTYPE_Icon = 3,						
			CELLTYPE_IconButton = 4,
			CELLTYPE_BigText = 5,		 		// Text with [...] button to see/edit whole thing.
			CELLTYPE_PopUp = 6,					// Popup list
			CELLTYPE_Color = 7					// Button with color sample and no text. 
			};
			
		enum FORMAT {
			FORMAT_Left =				0, //!< Left-align values
			FORMAT_Right =				1, //!< Right-align values
			FORMAT_Center =			2  //!< Center values in column
			};

		enum STYLE {
			STYLE_AlignTop =			0x0000,  //!<
			STYLE_NoGridLines =		0x0001,	//!< Don't show grid lines
			STYLE_MultiSelect =		0x0002,	//!< Allow multiple selection
			STYLE_SingleSel =			0x0004,	//!< Allow only single selection (default)
			STYLE_ShowSelAlways =	0x0008,	//!< Show selection even when don't have focus
			STYLE_SortAscending =	0x0010,	//!< Sort in ascending order
			STYLE_SortDescending =	0x0020,	//!< Sort in descending order
			STYLE_NoSelect =			0x0040,	//!< Don't allow selection
			STYLE_NoOuterFrame =		0x0080,	//!< Don't include outer frame
			STYLE_AutoArrange =		0x0100,  //!<
			STYLE_EditLabels =		0x0200,	//!< Allow editings
			STYLE_SizeInPixels =		0x0400,	//!< Sizes parameters for Create() specified in pixels
			STYLE_AlignLeft =			0x0800,	//!<
			STYLE_OwnerData =			0x1000,	//!<
			STYLE_NoScroll =			0x2000,	//!< Don't show scroll bars
			STYLE_NoColumnHeader =	0x4000,	//!< Don't show column headers
			STYLE_NoSortHeader =		0x8000,	//!< Don't allow sorting by pressing header
			};

		enum VIEWMODE {
			VIEWMODE_Icon =			0,		//!<LVS_ICON,				NOTE: Not implemented in X case
			VIEWMODE_Report =			1,		//!<LVS_REPORT,			
			VIEWMODE_SmallIcon =		2,		//!<LVS_SMALLICON		NOTE: Not implemented in X case
			VIEWMODE_List =			3		//!<LVS_LIST,				
			};

		enum IMAGELISTTYPE {
			IMAGELISTTYPE_Normal	=  0,		//!< LVSIL_NORMAL
			IMAGELISTTYPE_Small	=  1,		//!< LVSIL_SMALL
			IMAGELISTTYPE_State	=  2		//!< LVSIL_STATE
			};

		//! Structure passed to OnGetDispInfo
		//! Note: for the present, only the m_text member is supported by X
		struct DISPINFO {
			DISPINFO () :
				m_indent(0),
				m_IconIndex(-1),
				m_StateImageIndex(-1),
				m_OverlayImageIndex(-1),
				m_bSelected(false),
				m_CellType(CELLTYPE_Text)
				{}
				
			~DISPINFO() {}

			MISTRING m_text;				//!< String to display
			int m_indent;					//!< Number of icon widths to indent (default == 0)
			int m_IconIndex;				//!< 0-based Index into IMAGELIST set by SetImageList(), -1 == none
			int m_StateImageIndex;		//!< Displayed next to the item's icon -1 = none.  
			int m_OverlayImageIndex;	//!< Drawn on top of item's icon. -1 = none.
			bool m_bSelected;	
			CELLTYPE m_CellType;			//!< Set from CTRL_GRID::CELLTYPE
			MISTRING m_ToolTipStr;		//!< String to display as a data tip, call MISTRING::Clear() to show no data tip.
			};
		
		//! Structure passed to OnDrawItem()
		struct CUSTOMDRAW {
			int row;
			int col;
			LRECT2D rect;
			COLOR BackgroundColor;
			COLOR TextColor;
			bool ChangedBackgroundColor;
			bool ChangedTextColor;
			MGD::CONTEXT *gc;
			};

		// Delegates
		typedef fastdelegate::FastDelegate<bool(int row, int col)> DELEGATE_ONBEGINLABELEDIT;
		typedef fastdelegate::FastDelegate<void(int startcol, int endcol)> DELEGATE_ONCHANGECOLUMNORDER;
		typedef fastdelegate::FastDelegate<void(int row, int col, KEYSTATE keystate)> DELEGATE_ONCLICK;
		typedef fastdelegate::FastDelegate<bool(CUSTOMDRAW& customdraw)> DELEGATE_ONDRAWITEM;
		typedef fastdelegate::FastDelegate<bool(int row, int col, MISTRING&)> DELEGATE_ONENDLABELEDIT;
		typedef fastdelegate::FastDelegate<bool(int row, int col, DISPINFO&)> DELEGATE_ONGETDISPINFO;
		typedef fastdelegate::FastDelegate<int(UINT32 row1, UINT32 row2, int col, bool bAscending)> DELEGATE_ONITEMCOMPARE;
		typedef fastdelegate::FastDelegate<void(int row, int col)> DELEGATE_ONITEMSELECT;
		typedef fastdelegate::FastDelegate<void(int row, int col)> DELEGATE_ONITEMUNSELECT;
		typedef fastdelegate::FastDelegate<void(int row, int col, const LPOINT2D&point, KEYSTATE keystate)> DELEGATE_ONRIGHTCLICK;
		typedef fastdelegate::FastDelegate<void(int col, int width)> DELEGATE_ONRESIZECOLUMN;
		typedef fastdelegate::FastDelegate<bool(MGUI::CTRL_TOOLTIP&, int row, int col)> DELEGATE_ONTOOLTIP;

		//! Constructor.
		CTRL_GRID (
			);

		//! Destructor.
		virtual ~CTRL_GRID (
			);

		//! Create the control.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			int height = 10,								//!< Height in rows,
			int width = 40,								//!< Width in 'em' characters
			VIEWMODE viewmode = VIEWMODE_Report,	//!< Initial viewing mode
			STYLE style = STYLE_AlignTop,				//!< List style
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_Expand
			);

		//! Delete all columns.
		void DeleteAllColumns (
			);

		//! Delete all items.
		void DeleteAllItems (
			);

		//! Delete specified column.
		void DeleteColumn (
			int col
			);

		//! Delete specified item.
		void DeleteItem (
			int itemnum
			);

		void DeleteItem (
			int itemnum,
			bool redrawnow
			);

		//! Delete specified item given assigned data value.
		void DeleteItemByData (
			ID id
			);

		//! Deselect all items.
		void DeselectAllItems (
			);

		//! Finish any edits, used if need to save changes.
		void EditFinish ();

		//! Begin editing specified item, starting with currently displayed value.
		//! @return true if successful, false if not.
		bool EditItem (
			int itemnum,
			int column = 0,
			bool selectall = true						//!< Select all existing text so typing will replace
			);

		//! Begin editing specified item, starting with specified string.
		//! @return true if successful, false if not.
		bool EditItem (
			int itemnum,
			int column,
			const MIUNICODE *initstr,						//!< String to replace with prior to editing
			bool selectall = true						//!< Select all existing text so typing will replace
			);

		//! Ensures that an item is visible.
		void EnsureVisible (
			int itemnum,
			bool PartialOK = false
			);

		//! Find item by matching specified data value as pointer.
		//! @return Item number or -1 if none found.
		int FindItemByData (
			const void* value
			) const;

		//! Find item by matching specified data value as ID.
		//! @return Item number or -1 if none found.
		int FindItemByData (
			ID data
			) const { return (FindItemByData((void*)data)); }

		//! Get the index (row number) of the bottom-most visible item.
		//! Note: Return value may be greater than number of rows of data
		//! if more rows are visible than there are data for.
		int GetBottomRow (
			) const { return (GetTopRow() + GetCountPerPage()); }

		//! Get the bounding rectangle of a single cell.
		bool GetCellRect (
			int itemnum,
			int col,
			LRECT2D& rect
			) const;

		//! Get number of columns in the list.
		int GetColumnCount (
			) const;

		//! Get the label for a column.
		//! @return true if successful, false otherwise
		bool GetColumnLabel (
			int column,
			MISTRING& label
			) const;

		//! Get the order of columns in the list.  
		//! Columns can be reordered by dragging them around if you call
		//! SetExtendedStyle() with LVS_EX_HEADERDRAGDROP
		ERRVALUE GetColumnOrderArray (
			SIMPLE_ARRAY<INT32>& array
			) const;

		//! Get the width of a column in pixels.
		int GetColumnWidth (
			int column
			) const;

		//! Get number of items that can fit vertically in the visible area of control (in report mode only).
		int GetCountPerPage (
			) const;

		//! Get height of heading in pixels.
		int GetHeadingHeight (
			) const;

		//! Get number of items in list.
		int GetItemCount (
			) const;

		//! Get data value associated with item.
		MGUI::ID GetItemData (
			int itemnum
			) const;

		MISTRING GetItemText (
			int itemnum,
			int subitem
			) const;

		//! Get next selected item in list.
		//! @return Index of next item or -1 if no more items.
		int GetNextSelectedItem (
			int startitem = -1							//!< Starting item for search, starting item is excluded
			) const;

		//! Get number of selected items.
		int GetSelectedCount (
			) const;

		//! Get the index (row number) of the top-most visible item
		int GetTopRow (
			) const;

		//! Insert new column with label from MISTRING;
		int InsertColumn (
			int colnum,										//!< Column number and subitem
			const MISTRING& string,						//!< String for column heading
			FORMAT format = FORMAT_Left,				//!< Column format
			int width = -1									//!< Width of column in PIXELS
			);

		//! Insert new column with label from MIUNICODE* string.
		//! Every cell in the column is a combo box.
		//! The menu parameter is an MISTRING of newline-seperated entries to
		//! use when populating the drop down menu.  Note that other than having
		//! a fixed set of options in a dropdown menu, this column type still
		//! returns its current value as text, not as anything numeric. 
		int InsertColumnComboBox (
			int colnum,										//!< Column number and subitem
			const MISTRING& label,						//!< String for column heading
			const MISTRING& menu,
			int width = -1									//!< Width of column in PIXELS
			);

		//! Insert new item.
		//! @return Item number of inserted item if successful, < 0 if error.
		int InsertItem (
			const MISTRING& string						//!< Item string
			);

		//! Insert new item at specified position.
		//! @return Item number of inserted item if successful, < 0 if error.
		int InsertItem (
			int pos,											//!< Item position in list
			const MISTRING& string						//!< Item string
			);

		//! Determine if specified item is currently "selected".
		//! @return 'true' if selected, 'false' if not.
		bool IsItemSelected (
			int itemnum
			) const;

		//! Force the control to repaint all items.
		void Redraw (
			bool updatenow = true
			);

		//! Disable redraw, incrementing disable count.
		void RedrawDisable (
			);

		//! Enable redraw, decrementing disable count.
		//! Will redraw control if was disabled and is no longer.
		void RedrawEnable (
			);

		//! Force the control to repaint a range of items.
		void RedrawItems (
			int firstitemnum,
			int lastitemnum,
			bool updatenow = true
			);

		//! Set column label.
		//! @return true if successful, false otherwise
		bool SetColumnLabel (
			int column,
			const MISTRING& label
			);

		//! Set the order of columns within the list.
		void SetColumnOrderArray (
			SIMPLE_ARRAY<INT32>& array
			);

		//! Set the width of a column.  The width can be in pixels or one of the
		//! constants LVSCW_AUTOSIZE or LVSCW_AUTOSIZE_USE_HEADER as described in
		//! LVM_SETCOLUMNWIDTH in the Platform SDK documentation.
		void SetColumnWidth (
			int column,
			int width
			);

		void SetDelegateOnBeginLabelEdit (
			DELEGATE_ONBEGINLABELEDIT DelegateOnBeginLabelEdit
			) { m_DelegateOnBeginLabelEdit = DelegateOnBeginLabelEdit; }

		void SetDelegateOnChangeColumnOrder (
			DELEGATE_ONCHANGECOLUMNORDER DelegateOnChangeColumnOrder
			) { m_DelegateOnChangeColumnOrder = DelegateOnChangeColumnOrder; }

		void SetDelegateOnClickColumn (
			DELEGATE_ONCLICK DelegateOnClickColumn
			) { m_DelegateOnClickColumn = DelegateOnClickColumn; }

		//! Set delegate to call when click on item.
		//! Will be called after any selection operations have been done.
		void SetDelegateOnClickItem (
			DELEGATE_ONCLICK DelegateOnClickItem
			) { m_DelegateOnClickItem = DelegateOnClickItem; }

		void SetDelegateOnDoubleClickItem (
			DELEGATE_ONCLICK DelegateOnDoubleClickItem
			) { m_DelegateOnDoubleClickItem = DelegateOnDoubleClickItem; }

		void SetDelegateOnDrawItem (
			DELEGATE_ONDRAWITEM DelegateOnDrawItem
			) { m_DelegateOnDrawItem = DelegateOnDrawItem; }

		void SetDelegateOnEndLabelEdit (
			DELEGATE_ONENDLABELEDIT DelegateOnEndLabelEdit
			) { m_DelegateOnEndLabelEdit = DelegateOnEndLabelEdit; }

		void SetDelegateOnGetDispInfo (
			DELEGATE_ONGETDISPINFO DelegateOnGetDispInfo
			) { m_DelegateOnGetDispInfo = DelegateOnGetDispInfo; }

		void SetDelegateOnItemCompare (
			DELEGATE_ONITEMCOMPARE DelegateOnItemCompare
			) { m_DelegateOnItemCompare = DelegateOnItemCompare; }

		void SetDelegateOnItemSelect (
			DELEGATE_ONITEMSELECT DelegateOnItemSelect
			) { m_DelegateOnItemSelect = DelegateOnItemSelect; }

		void SetDelegateOnItemUnselect (
			DELEGATE_ONITEMUNSELECT DelegateOnItemUnselect
			) { m_DelegateOnItemUnselect = DelegateOnItemUnselect; }

		void SetDelegateOnResizeColumn (
			DELEGATE_ONRESIZECOLUMN DelegateOnResizeColumn
			) { m_DelegateOnResizeColumn = DelegateOnResizeColumn; }

		void SetDelegateOnRightClick (
			DELEGATE_ONRIGHTCLICK DelegateOnRightClick
			) { m_DelegateOnRightClick = DelegateOnRightClick; }

		void SetDelegateOnSize (
			DELEGATE_ONSIZE DelegateOnSize
			) { m_DelegateOnSize = DelegateOnSize; }

		void SetDelegateOnToolTip (
			DELEGATE_ONTOOLTIP DelegateOnToolTip
			) { m_DelegateOnToolTip = DelegateOnToolTip; }

		//! Set image list.
		void SetImageList (
			IMAGELIST& imagelist,						//!< Image list
			IMAGELISTTYPE listtype = IMAGELISTTYPE_Small,				//!< Type of image list, IMAGELISTTYPE_Normal, IMAGELISTTYPE_Small or IMAGELISTTYPE_State
			bool shared = false							//!< 'true' if image list is shared, 'false' if control should take ownership
			);

		//! Set number of items in list.
		int SetItemCount (
			int NumItems
			);

		//! Set application-specific data to associate with list item.
		void SetItemData (
			int itemnum,									//!< Item number
			ID value									//!< Value to associate with item
			);

		//! Set the image index to use for a givem item.
		//! Pass -1 for imageindex to have no icon
		//! imageindex is a 0-based index into the IMAGELIST set via
		//! SetImageList()
		void SetItemImage (
			int itemnum,
			int imageindex
			);

		//! Set whether item is 'selected or not.
		void SetItemSelected (
			int itemnum,									//!< Item number
			bool selected = true
			);

		//! Set item or subitem text.
		void SetItemText (
			int itemnum,									//!< Item number
			int subitem,									//!< Sub item, 0 for item
			const MISTRING& string						//!< Item or subitem string
			);

		//! Set the height of the rows in pixels
		void SetRowHeight (
			int pixels
			);

		//! Sort the data in the list on a given column.
		void SortItems (
			int column = 0,			//!< Column to sort on
			bool bAscending = true
			);

		//! Force the control to repaint a specified item
		void Update (
			int itemnum
			);

	protected:

		STYLE GetStyle (
			) const { return m_style; }

		//! Called when user clicks on label to begin editing.
		//! @return 'false' if editing is NOT allowed or 'true' if editing is allowed.
		//! Default implementation always returns 'true' if the grid was
		//! created with STYLE_EditLabels or 'false' if not.
		virtual bool OnBeginLabelEdit (
			int itemnum,
			int subitemnum
			);

		//! Called when the order of the columns are changed.
		virtual void OnChangeColumnOrder (
			int fromCol,
			int toCol
			);

		//! Called when user clicks on column heading.
		virtual void OnClickColumn (
			int itemnum,
			int column,
			KEYSTATE state
			);

		//! Called when user left-clicks on a list item.
		virtual void OnClickItem (
			int itemnum,															
			int column,
			KEYSTATE state
			);

		//! Called when user double-left-clicks on a list item.
		virtual void OnDoubleClickItem (
			int itemnum,
			int column,
			KEYSTATE state
			);
		
		//! Called before an item is drawn.
		//! Override if you want to do the drawing yourself. 
		//! Should return true if you did all the drawing.  Return false to let the base class draw the item.
		virtual bool OnDrawItem (
			CUSTOMDRAW& drawinfo
			);

		//! Called when user clicks on label to begin editing.
		//! @return 'true' if to accept changes or 'false' to reject changes.
		//! Default implementation always returns 'true' if the grid was
		//! created with STYLE_EditLabels or 'false' if not.
		virtual bool OnEndLabelEdit (
			int row,
			int col,
			MISTRING& string
			);

		//! Called by list to request information stored by owner rather than list.
		//!
		//! Derived implementation should return true if it actually filled in
		//! anything.  Return false to behave as if OnGetDispInfo wasn't overridden.
		virtual bool OnGetDispInfo (
			int row,
			int col,
			CTRL_GRID::DISPINFO& dispinfo 
			);

		//! Called when an item is "activated" (normally via a double click).
		virtual void OnItemActivate (
			);

		//! Called by SortItems() to compare items.
		//! Should return < 0 if rowitem1 should precede rowitem2 and > 0 if rowitem1 should follow rowitem2
		//! and 0 if the two items are equivalent.
		//! if bAscending is false, you should reverse the order.
		virtual int OnItemCompare (
			UINT32 rowitem1,
			UINT32 rowitem2,
			int colnum,
			bool bAscending
			);

		//! Called when an item is "selected".
		virtual void OnItemSelect (
			int itemnum,
			int subitemnum
			);

		//! Called when an item is "unselected".
		virtual void OnItemUnselect (
			int itemnum,
			int subitemnum
			);

		//! Called for owner-data list to provide hints about what to cache.
		virtual void OnOwnerDataCacheHint (
			int from,
			int to
			);

		//! Called when user right-clicks on a list item.
		virtual void OnRightClickItem (
			int itemnum,
			int column,
			const LPOINT2D&,
			KEYSTATE state
			);

		//! Called when user release mouse after dragging column header.
		virtual void OnUserResizeColumn (
			int column,
			int width
			);

		//! Called to get tooltip for a cell.
		virtual bool v_OnToolTip (
			MGUI::CTRL_TOOLTIP& tooltip,
			int row,
			int col
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		class MyListCtrl;
		friend class MyListCtrl;

		MyListCtrl *m_pCtrl;
		STYLE m_style;
		int m_PrevSelRow;
		int m_PrevSelCol;

		DELEGATE_ONBEGINLABELEDIT m_DelegateOnBeginLabelEdit;
		DELEGATE_ONCHANGECOLUMNORDER m_DelegateOnChangeColumnOrder;
		DELEGATE_ONCLICK m_DelegateOnClickColumn;
		DELEGATE_ONCLICK m_DelegateOnClickItem;
		DELEGATE_ONCLICK m_DelegateOnDoubleClickItem;
		DELEGATE_ONDRAWITEM m_DelegateOnDrawItem;
		DELEGATE_ONENDLABELEDIT m_DelegateOnEndLabelEdit;
		DELEGATE_ONGETDISPINFO m_DelegateOnGetDispInfo;
		DELEGATE_ONITEMCOMPARE m_DelegateOnItemCompare;
		DELEGATE_ONITEMSELECT m_DelegateOnItemSelect;
		DELEGATE_ONITEMUNSELECT m_DelegateOnItemUnselect;
		DELEGATE_ONRIGHTCLICK m_DelegateOnRightClick;
		DELEGATE_ONRESIZECOLUMN m_DelegateOnResizeColumn;
		DELEGATE_ONSIZE m_DelegateOnSize;
		DELEGATE_ONTOOLTIP m_DelegateOnToolTip;

		CTRL_GRID (const CTRL_GRID&);
		CTRL_GRID& operator= (const CTRL_GRID&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(MGUI::CTRL_GRID::STYLE)
#endif

//------------------------------------------------------------------------------------------------

}	// End of MGUI namespace

#endif	// INC_MGUI_GRID_H

