/**
 * \file <mgui/icon.h>
 * \brief MGUI::ICON definitions
 *
 * \if NODOC
 * $Id: icon.h_v 1.4 2011/08/22 13:47:13 mju Exp $
 *
 * $Log: icon.h_v $
 * Revision 1.4  2011/08/22 13:47:13  mju
 * Add updatePixmap.
 *
 * Revision 1.3  2010/03/24 15:04:57  mju
 * Add isEmpty.
 *
 * Revision 1.2  2010/03/05 21:16:24  mju
 * Add draw() method.
 *
 * Revision 1.1  2010/03/01 18:59:35  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_ICON_H
#define	INC_MGUI_ICON_H

#ifndef	INC_MI32_COLOR_H
#include <mi32/color.h>
#endif

#ifndef	INC_MI32_ICONID_H
#include <mi32/iconid.h>
#endif

#ifndef	INC_MI32_SIMPLEAR_H
#include <mi32/simplear.h>
#endif

#ifndef	_XLIB_H_
	#include <X11/Xlib.h>
#endif

namespace MGD {
	class CONTEXT;
	}

namespace MGUI {

//=====================================================================================================================

//! Icon bitmap.
class ICON {
	public:

		//! Drawing style.
		enum STYLE {
			STYLE_Normal = 0,					//!< Normal style
			STYLE_Armed,						//!< "Armed" (typically darker background)
			STYLE_Disabled,					//!< "Disabled" (grayed out)
			STYLE_DisabledArmed,				//!< Disabled and armed (grayed with darker background)
			STYLE_NoDrawBackground			//!< Don't draw background pixels (for Draw method only)
			};

		//! Orientation relative to original design.
		enum ORIENTATION {
			ORIENTATION_Upright = 0,		//!< Upright
			ORIENTATION_RotateCW,			//!< Rotate clockwise 90 degrees
			ORIENTATION_RotateCCW,			//!< Rotate counterclockwise 90 degrees
			ORIENTATION_Rotate180,			//!< Rotate 180 degrees
			ORIENTATION_MirrorHorizontal,	//!< Mirror in horizontal direction
			ORIENTATION_MirrorVertical,	//!< Mirror in vertical direction
			ORIENTATION_MirrorRotateCW,	//!< Mirror horizontally then rotate clockwise
			ORIENTATION_MirrorRotateCCW,	//!< Mirror horizontally then rotate counterclockwise
			};

		//! Default constructor, creates empty icon.
		ICON ();

		//! Copy constructor.
		//! Icons are partially reference-counted so copy and return-by-value is efficient.
		ICON (const ICON& rhs);

		//! Construct using ICONID with common background color, optional size and orientation.
		ICON (
			ICONID iconid,
			UINT16 width = 0,						//!< Width in pixels, 0 to use as designed
			UINT16 height = 0,					//!< Height in pixels, 0 to use as designed
			INT16 MarginLeft = -1,				//!< Left margin in pixels, -1 to center in specified width
			INT16 MarginTop = -1,				//!< Top margin in pixels, -1 to center in specified height
			ORIENTATION orientation = ORIENTATION_Upright	//!< Orientation
			);

		//! Construct using ICONID and background color, with optional size and orientation.
		ICON (
			ICONID iconid,
			const COLOR& ColorBg,				//!< Background color
			UINT16 width = 0,						//!< Width in pixels, 0 to use as designed
			UINT16 height = 0,					//!< Height in pixels, 0 to use as designed
			INT16 MarginLeft = -1,				//!< Left margin in pixels, -1 to center in specified width
			INT16 MarginTop = -1,				//!< Top margin in pixels, -1 to center in specified height
			ORIENTATION orientation = ORIENTATION_Upright	//!< Orientation
			);

		~ICON ();

		//! Assignment.
		//! Icons are partially reference-counted so assignment is relatively efficient.
		ICON& operator= (const ICON& rhs);

	#ifdef X_NATIVE
		//! Create X Pixmap for icon in specified style.
		//! @returned Pixmap, caller is responsible for freeing.
		Pixmap CreatePixmap (
			Widget widget,							//!< Widget for display/screen access
			STYLE style								//!< Icon style
			) const;
	#endif

		//! Draw icon.
		void Draw (
			MGD::CONTEXT& gc,
			INT32 x,									//!< X coordinate corresponding to left edge of icon
			INT32 y,									//!< Y coordinate corresponding to top edge of icon
			STYLE style = STYLE_Normal			//!< Icon style
			) const;

		//! Get icon height.
		UINT16 GetHeight () const
			{ return (m_SetHeight); }

		//! Get icon width.
		UINT16 GetWidth () const
			{ return (m_SetWidth); }

		//! Determine if icon is empty.
		bool IsEmpty () const;

		//! Set background color.
		void SetColorBackground (
			const COLOR& ColorBg					//!< Color for both "normal" and "armed" background
			);

		//! Set background colors for normal and "armed" states..
		void SetColorBackground (
			const COLOR& ColorBgNormal,		//!< Color to use for normal background
			const COLOR& ColorBgArmed			//!< Color to use for "armed" background
			);

		//! Set "variable" color.
		//! An icon may have one or more "variable" colors that can be set at runtime.
		//! Attempts to set for index that is out of range will be ignored.
		void SetColorVar (
			const COLOR& color,
			UINT8 var = 0							//!< Variable color index
			);

		//! Set orientation.
		//! Orientation is relative to original design, so changes are not cumulative.
		void SetOrientation (
			ORIENTATION orientation				//<! Orientation relative to original design
			) { m_Orientation = orientation; }

		//! Set icon size.
		void SetSize (
			UINT16 width,							//!< Width in pixels, if less than design width will use as designed
			UINT16 height,							//!< Height in pixels, if less than design height will use as designed
			INT16 MarginLeft = -1,				//!< Left margin in pixels, -1 to center in specified width
			INT16 MarginTop = -1					//!< Top margin in pixels, -1 to center in specified height
			);

	#ifdef X_NATIVE
		//! Update X Pixmap for icon in specified style.
		void UpdatePixmap (
			Widget widget,							//!< Widget for display/screen access
			Pixmap pixmap,							//!< Pixmap to update, must be correct size
			STYLE style								//!< Icon style
			) const;
	#endif

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;
		UINT16 m_SetWidth;
		UINT16 m_SetHeight;
		INT16 m_MarginLeft;
		INT16 m_MarginTop;
		ORIENTATION m_Orientation;
		COLOR m_ColorBgNormal;
		COLOR m_ColorBgArmed;
		SIMPLE_ARRAY<COLOR> m_ColorVar;

		void GetCellBGRA (INT16 x, INT16 y, UINT8 *bgra) const;
		void SetupColorVar ();
		#endif
	};

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_ICON_H
