/**
 * \file mi32/delegate.h
 * \brief Common delegate type definitions.
 *
 * A delegate is a type-safe way to call a function without the caller knowing
 * anything about the whether the function being called is a class method, a
 * virtual method or a static function.  This allows classes to be loosely
 * coupled without use of virtual overrides.  One common example of this is
 * in user interface controls where a 'containing' class such as a form or
 * dialog has functions to be called based on interaction with controls contained
 * in the dialog.  Instead of requiring a virtual override for each control, a
 * delegate may be set when the control is created which will call a function in
 * the containing class.
 *
 * A delegate instance may be constructed for any function which matches the
 * delegate signature, including virtual, non-virtual and static class methods
 * as well as global functions.  Construction for virtual and non-virtual
 * class methods also requires the pointer to the class instance.
 *
 * For example:
 *		\code
 *
 *		// Class with methods to set delegates.
 *		class SOMECLASS {
 *			public:
 *				void SetDelegateGetSomething (
 *					DELEGATE_DOUBLE_NOPARMS delegate
 *					) { m_DelegateGetSomething = delegate; }
 *				void SetDelegateNotifySomething (
 *					DELEGATE_VOID_NOPARMS delegate
 *					) { m_DelegateNotifySomething = delegate; }
 *			private:
 *				DELEGATE_DOUBLE_NOPARMS m_DelegateGetSomething;
 *				DELEGATE_VOID_NOPARMS m_DelegateNotifySomething;
 *				// Invoke DelegateGetSomething (would be called internally to class)
 *				double GetSomething (
 *					) { if (m_DelegateGetSomething) return (m_DelegateGetSomething(); else return (0.0); }
 *				// Invoke DelegateNotifySomething (would be called internally to class)
 *				void NotifySomething (
 *					) { if (m_DelegateNotifySomething) m_DelegateNotifySomething(); }
 *			};
 *			
 *		// Class containing delegate methods to call.
 *		class TARGETCLASS {
 *			public:
 *				void Create (
 *					) {
 *					m_SomeClass.SetDelegateGetSomething(DELEGATE_DOUBLE_NOPARMS(&TARGETCLASS::GetSomething));
 *					m_SomeClass.SetDelegateNotifySomething(DELEGATE_VOID_NOPARMS(this,&TARGETCLASS::OnNotifySomething));
 *					}
 *			private:
 *				SOMECLASS m_SomeClass;
 *				// Static method returning double.
 *				static double GetSomething ();
 *				// Normal method returning void.
 *				void OnNotifySomething ();
 *			};
 *
 *		// Global function returning double.
 *		static double GlobalGetSomething ();
 *
 *		void MyFunc (
 *			) {
 *			SOMECLASS SomeClass;
 *			SomeClass.SetDelegateGetSomething(&GlobalGetSomething);
 *			...
 *			}
 *
 *		\endcode
 *
 * \if NODOC
 * $Id: delegate.h_v 1.13 2012/06/04 20:35:24 mju Exp $
 *
 * $Log: delegate.h_v $
 * Revision 1.13  2012/06/04 20:35:24  mju
 * Add delegate_errvalue_bool.
 *
 * Revision 1.12  2012/05/10 16:58:37  mju
 * Add delegate_objitem_noparms.
 *
 * Revision 1.11  2011/10/10 16:20:05  mju
 * Add delegate_errvalue_polyline.
 *
 * Revision 1.10  2011/06/27 15:47:18  mju
 * Add delegate_errvalue_double.
 *
 * Revision 1.9  2008/02/11 15:05:14  mju
 * Add delegate_void_dpoint2d.
 *
 * Revision 1.8  2007/04/02 14:45:28  mju
 * Add delegate_void_int32.
 *
 * Revision 1.7  2007/03/26 14:29:34  dwilliss
 * added void no bool
 *
 * Revision 1.6  2006/10/13 19:52:49  dwilliss
 * needed to forward declare mistring
 *
 * Revision 1.5  2006/10/13 19:42:18  dwilliss
 * Added DELEGATE_VOID_MISTRING
 *
 * Revision 1.4  2006/06/01 17:38:59  dwilliss
 * Added a couple new typedefs
 *
 * Revision 1.3  2005/06/23 14:53:39  mju
 * Add general documentation.
 *
 * Revision 1.2  2005/06/22 21:57:08  mju
 * Add typedef for delegate returning double.
 *
 * Revision 1.1  2005/06/22 15:29:48  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_DELEGATE_H
#define	INC_MI32_DELEGATE_H

#ifndef INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifndef FASTDELEGATE_H
#include <mi32/fastdelegate.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
struct DPOINT2D;
class MISTRING;
class POLYLINE;
namespace RVC {
	class OBJITEM;
	}
#endif

using namespace fastdelegate;

// Define commonly used forms.

//! Delegate for function returning void and having no parameters.
typedef fastdelegate::FastDelegate<void()> DELEGATE_VOID_NOPARMS;

//! Delegate for function returning void and taking a bool parameter.
typedef fastdelegate::FastDelegate<void(bool)> DELEGATE_VOID_BOOL;

//! Delegate for function returning void and taking a INT32 parameter.
typedef fastdelegate::FastDelegate<void(INT32)> DELEGATE_VOID_INT32;

//! Delegate for function returning void and taking a const MISTRING reference
typedef fastdelegate::FastDelegate<void(const MISTRING&)> DELEGATE_VOID_MISTRING;

//! Delegate for function returning void and taking a const DPOINT2D reference
typedef fastdelegate::FastDelegate<void(const DPOINT2D&)> DELEGATE_VOID_DPOINT2D;

//! Delegate for function returning bool and having no parameters.
typedef fastdelegate::FastDelegate<bool()> DELEGATE_BOOL_NOPARMS;

//! Delegate for function returning ERRVALUE and having no parameters.
typedef fastdelegate::FastDelegate<ERRVALUE()> DELEGATE_ERRVALUE_NOPARMS;

//! Delegate for function returning ERRVALUE and having "bool" parameter.
typedef fastdelegate::FastDelegate<ERRVALUE(bool)> DELEGATE_ERRVALUE_BOOL;

//! Delegate for function returning ERRVALUE and having "double" parameter.
//! Typically used for progress reporting.
typedef fastdelegate::FastDelegate<ERRVALUE(double)> DELEGATE_ERRVALUE_DOUBLE;

//! Delegate for function returning double and having no parameters.
typedef fastdelegate::FastDelegate<double()> DELEGATE_DOUBLE_NOPARMS;

//! Delegate for function returning bool and taking an INT32.
typedef fastdelegate::FastDelegate<bool(INT32)> DELEGATE_BOOL_INT32;

//! Delegate for function returning bool and taking a UINT32.
typedef fastdelegate::FastDelegate<bool(UINT32)> DELEGATE_BOOL_UINT32;

//! Delegate for function returning ERRVALUE and taking a const POLYLINE reference
typedef fastdelegate::FastDelegate<ERRVALUE(const POLYLINE&)> DELEGATE_ERRVALUE_POLYLINE;

//! Delegate for function returning const RVC::OBJITEM reference and taking no parameters.
typedef fastdelegate::FastDelegate<const RVC::OBJITEM&()> DELEGATE_OBJITEM_NOPARMS;

#endif	// INC_MI32_DELEGATE_H
