/**
 *	\file mi32/geom2d.h
 * \brief Functions for 2-D geometry.
 *
 * \if NODOC
 * $Id: geom2d.h_v 1.98 2012/01/12 18:56:22 mju Exp $
 *
 * $Log: geom2d.h_v $
 * Revision 1.98  2012/01/12 18:56:22  mju
 * Remove unused extendLineToRectD fn.
 *
 * Revision 1.97  2011/11/09 15:11:57  mju
 * Fix dox warning.
 *
 * Revision 1.78  2001/02/21 14:30:16  mju
 * \endif
**/

#ifndef	INC_MI32_GEOM2D_H
#define	INC_MI32_GEOM2D_H

#ifndef	INC_MI32_POINT_H
#include <mi32/point.h>
#endif

#ifdef GEOMDLL
	#define GEOMLIBEXPORT MI_DLLEXPORT
#else
	#define GEOMLIBEXPORT MI_DLLIMPORT
#endif

struct DRECT2D;

#if defined(__cplusplus)
extern "C" {
#endif

//-------------------------------------------------------------------------------------------------
//!		Spline function flags																	
//-------------------------------------------------------------------------------------------------

#define SPLINE_UseMinDistance	0x0001
#define SPLINE_TreatAsClosed	0x0002
#define SPLINE_DontMoveEnds	0x0004

#define SPLINETYPE_None				0
#define SPLINETYPE_Cubic			1
#define SPLINETYPE_Quad				2
#define SPLINETYPE_Bezier			3

//!:Associate with "Geometric Functions (2D)"
//!\addtogroup geom2d Geometric Functions (2D)
//!@{

//! Determine point location relative to a line (double coordinates).
//!
//! @return -1 if left of line, 0 if on line, +1 if right of line.
//!
//! A right-handed cartesian coordinate system is assumed for left/right determination.
GEOMLIBEXPORT int CheckPointD (
	const DPOINT2D *start, 					//!<	Starting point used to define line
	const DPOINT2D *end, 					//!<	Ending point used to define line
	const DPOINT2D *point					//!< Point to check
	);

//! Determine point location relative to a line (LONG coordinates).
//!
//! @return -1 if left of line, 0 if on line, +1 if right of line.
//!
//! A right-handed cartesian coordinate system is assumed for left/right determination.
GEOMLIBEXPORT int CheckPointL (
	const LPOINT2D *start, 					//!< Starting point used to define line
	const LPOINT2D *end, 					//!< Ending point used to define line
	const LPOINT2D *point					//!< Point to check
	);

//! Clip line to rectangle (double coordinates).
//!
//! @return 0 - line completely outside cliprect, 1 - line start was modifed, 2 - line end was modified,
//!			  3 - both ends were modified, 4 - line completely inside cliprect.
int ClipLineD (
	const DRECT2D *cliprect, 				//!< Rectangle to clip to
	DPOINT2D *start, 							//!< Starting point of segment, modified if necessary
	DPOINT2D *end								//!< Ending point of segment, modified if necessary
	);

//! Compute buffer on one side of a polyline.
//!
//! Note: Consecutive points in the input point list MUST be different.
//!	Positive buffer distances will result in a buffer to the right of the line
//!	assuming Cartesian coordinates (+ right/up, - left/down).
ERRVALUE ComputeSimpleBufferL (
	const LPOINT2D *ipointlist, 
	int inumpoints, 
	double bufdist, 
	int joinstyle, 
	LPOINT2D **rpointlist, 					//!< Allocated point list returned
	int *rnumpoints							//!< Number of points returned
	);

//! Determine distance between point and line segment (double coordinates).
//!
//! @return Distance to closest point on line segment.
GEOMLIBEXPORT double DistPointLineSegD (
	const DPOINT2D *start, 					//!< Starting point of line segment
	const DPOINT2D *end, 					//!< Ending point of line segment
	const DPOINT2D *point, 					//!< Point to test
	DPOINT2D *closest							//!< Closest point on segment returned (NULL if not needed)
	);

//! Determine distance between point and line segment (WORD coordinates).
//!
//! @return Distance to closest point on line segment.
GEOMLIBEXPORT double DistPointLineSegW (
	const WPOINT2D *start, 					//!< Starting point of line segment
	const WPOINT2D *end, 					//!< Ending point of line segment
	const WPOINT2D *point, 					//!< Point to test
	DPOINT2D *closest							//!< Closest point on segment returned (NULL if not needed, DPOINT2D)
	);

//! Determine distance between point and rectangle (double coordinates).
//!
//! @return Distance to closest point on rectangle.
GEOMLIBEXPORT double DistPointRectD (
	const DRECT2D *rect, 					//!< Rectangle to compute distance to
	const DPOINT2D *point, 					//!< Point to test
	DPOINT2D *closest							//!< Closest point on rectangle returned (NULL if not needed)
	);

//! Determine if line segments intersect, where, and type of intersection.
//! @return Type and location of intersection.
GEOMLIBEXPORT int FindLineInt (
	const DPOINT2D *line1pt1, 
	const DPOINT2D *line1pt2, 
	const DPOINT2D *line2pt1, 
	const DPOINT2D *line2pt2, 
	DPOINT2D *intpoint
	);

//!	Compute cubic B - spline for given point list / 2D - line.
//!
//! @return ret > 0 - number of points in output buffer, ret == 0 - too few lines in input buffer, can't compute spline, but this is not an error. ret < 0 - error code.
//!
//! Processing flags:
//! 	SPLINE_UseMinDistance - use Tolerance parameter
//! 	SPLINE_TreatAsClosed  - all non-closed lines will be treated as closed, this flag has no impact on really closed lines.
//! Note: Programmer can use the same pointers for input and output buffers.
//!	IMPORTANT: Tolerance value will be ALWAYS used to eliminate dup. points!
INT32 MfitPointsBSpline (
	DPOINT2D *In, 								//!< Input array of points - passed
	DPOINT2D **Out, 							//!< Pointer to output array - returned
	INT32 NumPts, 								//!< Number of points in input array
	int F, 										//!< Number of additional knots between input points
	double Tolerance, 						//!< Currently used only with SPLINE_UseMinDistance flag minimal allowed distance between points in spline
	UINT32 Flags								//!< Flags	
	);

//!	Compute quadratic B - spline for given point list / 2D - line.
//!
//! @return ret > 0 - number of points in output buffer, ret == 0 - too few lines in input buffer, can't compute spline, but this is not an error. ret < 0 - error code.
//!
//! Note: Programmer can use the same pointers for input and output buffers.
//! IMPORTANT: Tolerance value will be ALWAYS used to eliminate dup. points!
//! Processing flags:	
//!		SPLINE_UseMinDistance - use Tolerance parameter
//!		SPLINE_TreatAsClosed  - all non-closed lines will be treated as closed, this flag has no impact on really closed lines.
INT32 MfitPointsBSplineQuadratic (
	DPOINT2D *In, 								//!< Input array of points - passed
	DPOINT2D **Out, 							//!< Pointer to output array - returned
	INT32 NumPts, 								//!< Number of points in input array
	int F, 										//!< Number of additional knots between input points.
	double Tolerance, 						//!< Currently used only with SPLINE_UseMinDistance flag minimal allowed distance between points in spline.
	UINT32 Flags								//!< Flags
	);

//!	Compute Bezier for given point list / 2D - line.
//!
//! @return ret > 0 - number of points in output buffer, ret == 0 - too few lines in input buffer, can't compute spline, but this is not an error. ret < 0 - error code.
//!
//! Note: Programmer can use the same pointers for input and output buffers.
//!	IMPORTANT: Tolerance value will be ALWAYS used to eliminate dup. points!
INT32 MfitPointsBezier (
	DPOINT2D *In, 								//!< Input array of points - passed
	DPOINT2D **Out, 							//!< Pointer to output array - returned
	INT32 NumPts, 								//!< Number of points in input array
	INT32 NumOut, 								//!< Number of output points
	double Tolerance, 						//!< Currently used only with SPLINE_UseMinDistance flag minimal allowed distance between points in spline
	UINT32 Flags								//!< Processing flags:	SPLINE_UseMinDistance - use Tolerance parameter
	);

//! Compute spline for given point list / 3D - line.
//!
//! @return ret > 0 - number of points in output buffer, ret == 0 - too few lines in input buffer, can't compute spline, but this is not an error. ret < 0 - error code.
//!
//!	NOTE: This function doesn't spline in XY-plane, it's just densify line and
//!	splines in Z/distance plane.
//!	Note: Programmer can use the same pointers for input and output buffers.
//!	IMPORTANT: Tolerance value will be ALWAYS used to eliminate dup. points!
//! Type of the spline to use:
//!		SPLINETYPE_Cubic - cubic spline (default type)
//!		SPLINETYPE_Quad  - quadratic spline
//!		SPLINETYPE_Bezier - Bezier spline
//! Processing flags:
//!		SPLINE_UseMinDistance - use Tolerance parameter
//!		SPLINE_TreatAsClosed  - all non-closed lines will be treated as closed, this flag has no impact on really closed lines.
INT32 MfitPointsSpline3D (
	DPOINT3D *In, 								//!< Input array of points - passed
	DPOINT3D **Out, 							//!<	Pointer to output array - returned
	INT32 NumPts, 								//!<	Number of points in input array
	INT32 F, 									//!<	Number of additional knots between input points or desired number of points in output (for Bezier)
	double Tolerance,							//!<	Currently used only with SPLINE_UseMinDistance flag minimal allowed distance between points in spline
	int SplineType, 							//!< Spline type
	UINT32 Flags								//!< Flags
	);
	
//!	Determine if a point is inside a polygon (double coordinates).
//!
//! @return 0 if point outside, 1 if point inside.
GEOMLIBEXPORT int PointInPolyD (
	const DPOLYGON *polygon, 				//!< Polygon to use
	const DPOINT2D *point					//!< Point to test
	);

//!	Determine if a point is inside a polygon (LONG coordinates).
//!
//! @return 0 if point outside, 1 if point inside.
GEOMLIBEXPORT int PointInPolyL (
	const LPOLYGON *polygon, 				//!< Polygon to use
	const LPOINT2D *point 					//!< Point to test
	);

//!	Solve general set of linear equations.
//!
//! Notes:	Solve the matrix equation:  A * X = B		for X
//!	      	Where:	A is an NxN matrix, X is a 1xN matrix, B is an Nx1 matrix
GEOMLIBEXPORT int SolveLinear (
	int n, 										//!< Number of equations
	double *A, 									//!< NxN coefficient matrix "A"
	double *b, 									//!< Nx1 result matrix "B"
	double *x									//!< 1xN unknown matrix returned "X"
	);

//! Solve general set of linear equations, using MatSolveSVD().
//!
//! Notes:	Solve the matrix equation:  A * X = B		for X
//!	      	Where:	A is an NxN matrix		       	
//!							X is a 1xN matrix
//! 	       			B is an Nx1 matrix
int SolveLinearSVD (
	int n, 										//!< Number of equations
	double *A, 									//!< NxN coefficient matrix "A"
	double *B, 									//!< Nx1 result matrix "B"
	double *X									//!< 1xN unknown matrix returned "X"
	);

#if defined(__cplusplus)
}
#endif

//!@}

#endif					//!<  #ifndef INC_MI32_GEOM2D_H 
