/**
 * \file mi32/gpsdbase.h
 *
 * \if NODOC
 * $Id: gpsdbase.h_v 1.11 2011/12/28 22:22:04 mju Exp $
 *
 * $Log: gpsdbase.h_v $
 * Revision 1.11  2011/12/28 22:22:04  mju
 * Remove redundant hdrs.
 *
 * Revision 1.10  2011/08/26 13:25:31  mju
 * Use formradiobuttons hdr.
 *
 * Revision 1.9  2008/08/26 15:31:29  bcolombe
 * Added directional fields and new date formats
 *
 * Revision 1.8  2007/03/07 22:00:15  bcolombe
 * Added DLG_TXTIMPORT for creating dialog to setup text log parameters
 *
 * Revision 1.7  2007/03/06 18:31:23  bcolombe
 * Added heading, speed and crs to logparms
 *
 * Revision 1.6  2007/03/02 16:09:06  bcolombe
 * Added format to logs
 *
 * Revision 1.5  2007/02/26 17:54:05  bcolombe
 * Added LOGPARMS class with DATEFORMAT and TIMEFORMAT
 * Used to hold settings passed to ReadLogText
 *
 * Revision 1.4  2007/02/23 17:22:26  bcolombe
 * Added LogParms class that will be used to setup paramaters for reading a log from a formatted text file
 *
 * Revision 1.3  2006/12/29 17:16:21  bcolombe
 * Fixed operator<
 *
 *
 * \endif
****************************************************************************/

#ifndef	INC_MI32_GPSDBASE_H
#define	INC_MI32_GPSDBASE_H

#ifndef INC_MI32_DATETIME_H 
#include <mi32/datetime.h>
#endif

#ifndef	INC_MI32_FILEPATH_H
#include <mi32/filepath.h>
#endif

#ifndef INC_MI32_SPATREF_H
#include <mi32/spatref.h>
#endif

#ifndef INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

#ifndef INC_MGUI_COMBOBOX_H
#include <mgui/combobox.h>
#endif

#ifndef INC_MGUI_FORMRADIOBUTTONS_H
#include <mgui/formradiobuttons.h>
#endif

#ifndef INC_MGUI_FORMBTXT_H
#include <mgui/formbtxt.h>
#endif

#ifndef	INC_VECTOR
#include <vector>
#define	INC_VECTOR
#endif


//! Class to compute a location based on Date and Time using GPS logs
class GPSDATABASE {
	public:
		//! Location Computing Methods
		enum GPSMETHOD {
			GPSMETHOD_Interpolate,
			GPSMETHOD_Closest
			};

		//! Default constructor
		GPSDATABASE ( );
		
		//! Default destructor
		~GPSDATABASE () { }

		//! Parms class for reading from formatted text file
		class LOGPARMS {
			public:
				enum DATEFORMAT {
					DATEFORMAT_YYYYMMDD,
					DATEFORMAT_YYYY_MM_DD,	//!< Contains a separator of some kind between values
					DATEFORMAT_MMDDYYYY,
					DATEFORMAT_MM_DD_YYYY,
					DATEFORMAT_DDMMYYYY,
					DATEFORMAT_DD_MM_YYYY,
					DATEFORMAT_YYMMDD,
					DATEFORMAT_YY_MM_DD,
					DATEFORMAT_MMDDYY,
					DATEFORMAT_MM_DD_YY,
					DATEFORMAT_DDMMYY,
					DATEFORMAT_DD_MM_YY
					};

				enum TIMEFORMAT {
					TIMEFORMAT_HHMMSS,
					TIMEFORMAT_HH_MM_SS		//!< Contains a separator of some kind between values
					};

				LOGPARMS(
					int XField,
					int YField,
					int ZField,
					int XDirField,
					int YDirField,
					int DateField,
					int TimeField,
					int HeadingField,
					int SpeedField,
					MISTRING delimiter,
					int NumHeaderLines = 0,
					DATEFORMAT DateFmt = DATEFORMAT_YYYYMMDD,
					TIMEFORMAT TimeFmt = TIMEFORMAT_HHMMSS,
					DMSTYPE LatLonFmt = DMSTYPE_DecDeg,
					bool IsColumnFormat = false
					);

				LOGPARMS();

				~LOGPARMS() {}

				int GetXField() const {return m_XField;}
				void SetXField(int XField) {m_XField = XField;}

				int GetYField() const {return m_YField;}
				void SetYField(int YField) {m_YField = YField;}

				int GetXDirField() const {return m_XDirField;}
				void SetXDirField(int XDirField) {m_XDirField = XDirField;}

				int GetYDirField() const {return m_YDirField;}
				void SetYDirField(int YDirField) {m_YDirField = YDirField;}

				int GetZField() const {return m_ZField;}
				void SetZField(int ZField) {m_ZField = ZField;}

				int GetDateField() const {return m_DateField;}
				void SetDateField(int DateField) {m_DateField = DateField;}

				int GetTimeField() const {return m_TimeField;}
				void SetTimeField(int TimeField) {m_TimeField = TimeField;}

				int GetHeadingField() const {return m_HeadingField;}
				void SetHeadingField(int HeadingField) {m_HeadingField = HeadingField;}

				int GetSpeedField() const {return m_SpeedField;}
				void SetSpeedField(int SpeedField) {m_SpeedField = SpeedField;}

				DATEFORMAT GetDateFmt() const {return m_DateFmt;}
				void SetDateFmt(DATEFORMAT DateFmt) {m_DateFmt = DateFmt;}

				TIMEFORMAT GetTimeFmt() const {return m_TimeFmt;}
				void SetTimeFmt(TIMEFORMAT TimeFmt) {m_TimeFmt = TimeFmt;}

				DMSTYPE GetLatLonFmt() const {return m_LatLonFmt;}
				void SetLatLonFmt(DMSTYPE LatLonFmt) {m_LatLonFmt = LatLonFmt;}

				int GetNumHeaderLines() const {return m_NumHeaderLines;}
				void SetNumHeaderLines(int NumHeaderLines) {m_NumHeaderLines = NumHeaderLines;}

				SPATREF::COORDREFSYS GetCrs() const {return m_crs;}
				void SetCrs(SPATREF::COORDREFSYS crs) {m_crs = crs;}

				const MISTRING& GetDelimiter () const {return m_Delimiter;}
				void SetDelimiter (MISTRING val) {m_Delimiter= val;}

				bool GetIsColumnFormat () const {return m_IsColumnFormat;}
				void SetIsColumnFormat (bool val) {m_IsColumnFormat = val;}

			private:
				int m_XField;						//!< X Coordinate Field number
				int m_YField;						//!< Y Coordinate Field number
				int m_ZField;						//!< Z Coordinate Field number
				int m_XDirField;					//!< X directional field number
				int m_YDirField;					//!< Y directional field number
				int m_DateField;					//!< Field number containing date
				int m_TimeField;					//!< Field number containing time
				int m_HeadingField;					//!< Field number containing heading
				int m_SpeedField;					//!< Field number containing speed
				DATEFORMAT m_DateFmt;				//!< Format of Date Field
				TIMEFORMAT m_TimeFmt;				//!< Format of Time Field
				DMSTYPE m_LatLonFmt;				//!< Format of Latitude/Longitude
				int m_NumHeaderLines;				//!< Number of Header Lines to skip
				SPATREF::COORDREFSYS m_crs;
				MISTRING m_Delimiter;
				bool m_IsColumnFormat;
			};

		//! Point Record
		class POINTRECORD {
			public:
				POINTRECORD();
				POINTRECORD(const POINTRECORD& rhs);
				~POINTRECORD() {}

				int GetLogNumber() const {return m_lognumber;}
				void SetLogNumber(int lognumber) {m_lognumber = lognumber;}

				DATETIME GetDateTime() const {return m_datetime;}
				void SetDateTime(const DATETIME& datetime) {m_datetime = datetime;}

				DPOINT3D GetLocation() const {return m_location;}
				void SetLocation(const DPOINT3D& location) {m_location = location;}

				double GetHeading() const {return m_heading;}
				void SetHeading(double heading) {m_heading = heading;}

				double GetSpeed() const {return m_speed;}
				void SetSpeed(double speed) {m_speed = speed;}

				bool operator< (const POINTRECORD& rhs ) const {return (m_datetime < rhs.m_datetime);}

			private:
				int m_lognumber;	//!< Number of Log Record came from
				DATETIME m_datetime;	//!< Record's Date/Time
				DPOINT3D m_location;	//!< Records' Lat/Lon location
				double m_heading;		//!< Heading in degrees
				double m_speed;		//!< Speed in m/s
			};

		//! Log Record
		class LOGRECORD {
			public:
				//! Format of logs
				enum LOGFORMAT {
					LOGFORMAT_GPX = 1,
					LOGFORMAT_NMEA = 2,
					LOGFORMAT_MI = 3,
					LOGFORMAT_TEXT = 4
					};

				LOGRECORD();
				LOGRECORD(const LOGRECORD& rhs);
				~LOGRECORD() {}

				LOGFORMAT GetLogFormat() const {return m_logformat;}
				void SetLogFormat(LOGFORMAT logfmt) {m_logformat = logfmt;}

				FILEPATH GetLogPath() const {return m_logpath;}
				void SetLogPath(const FILEPATH& logpath) {m_logpath = logpath;}

				MISTRING GetLogName() const {return m_logname;}
				void SetLogName(const MISTRING& logname) {m_logname = logname;}

				int GetLogNumber() const {return m_lognumber;}
				void SetLogNumber(int lognumber) {m_lognumber = lognumber;}

				DATETIME GetStartDateTime() const {return m_startdatetime;}
				void SetStartDateTime(const DATETIME& datetime) {m_startdatetime = datetime;}
				DATETIME GetEndDateTime() const {return m_enddatetime;}
				void SetEndDateTime(const DATETIME& datetime) {m_enddatetime = datetime;}

			private:
				FILEPATH m_logpath;		//!< Filepath of log
				MISTRING m_logname;		//!< "Name" of log
				int m_lognumber;		//!< Log's Number
				LOGFORMAT m_logformat;		//!< Format of log

				DATETIME m_startdatetime;	//!< Date/Time of log's first record
				DATETIME m_enddatetime;	//!< Date/Time of log's last record
			};

		#ifdef USE_MGUI
		//! DLGSHELL for setting parameters to read formatted text log file
		class DLG_IMPORTTXT : public MGUI::DLGSHELL {
			public:
				DLG_IMPORTTXT ();
				virtual ~DLG_IMPORTTXT ();

				GPSDATABASE::LOGPARMS GetParms() const {return m_parms;}
				bool GetOkPressed() const {return m_OkPressed;}

			private:
				GPSDATABASE::LOGPARMS m_parms;
				SPATREF::COORDREFSYS m_crs;
				// Create dialog contents.
				MGUI::FORM_EDIT_NUMBER m_NumHeaderLines;
				MGUI::FORM_EDIT_NUMBER m_XField;
				MGUI::FORM_EDIT_NUMBER m_YField;
				MGUI::FORM_EDIT_NUMBER m_XDirField;
				MGUI::FORM_EDIT_NUMBER m_YDirField;
				MGUI::FORM_EDIT_NUMBER m_ZField;
				MGUI::FORM_EDIT_NUMBER m_DateField;
				MGUI::FORM_EDIT_NUMBER m_TimeField;
				MGUI::FORM_EDIT_NUMBER m_HeadingField;
				MGUI::FORM_EDIT_NUMBER m_SpeedField;
				MGUI::FORM_COMBOBOX m_LatLonFmt;
				MGUI::FORM_COMBOBOX m_DateFmt;
				MGUI::FORM_COMBOBOX m_TimeFmt;
				MGUI::FORM_RADIOBUTTONS m_IsColumn;
				MGUI::FORM_EDIT_STRING m_DelimitedByEdit;
				MGUI::FORM_BUTTONTEXT m_FormCRS;
				bool m_OkPressed;
				void OnSetFormat();
				void OnSelectCRS();

				virtual ERRVALUE v_CreateContent();
				virtual void v_OnOK();
				virtual void v_OnCancel();
			};
		#endif

		//! Set the camera offset time in seconds
		void SetOffset (INT32 offset);

		//! Adjusts a DATETIME by current offset (0 if not set)
		DATETIME& ApplyOffset(DATETIME datetime); 

		//! Read a GPS Log file, add records to list
		ERRVALUE ReadLog (const FILEPATH& logpath);		//!< FILEPATH of log to read

		//! Read a formatted text GPS log file
		ERRVALUE ReadLogText (
			const FILEPATH& logpath,		//!< FILEPATH of log to read
			const LOGPARMS& logparms		//!< LOGPARMS used to determine how to properly parse file
			);
			
		//! Compute the location given Date/Time
		void Compute (
			const DATETIME& datetime,					//!< DateTime to locate for
			POINTRECORD& OutputRecord,					//!< Record returned
			POINTRECORD& PrevRecord,						//!< Record found before DateTime
			POINTRECORD& NextRecord,						//!< Record found after DateTime
			GPSMETHOD method = GPSMETHOD_Interpolate,	//!< Searching method
			double errortime = 60.0						//!< Seconds, Difference in time between input and records must fall within errortime
			);

		//! Add a log given a FILEPATH
		//! Does not parse like ReadGPS, only adds to LogList
		ERRVALUE AddLogPath (
			const FILEPATH& logpath,
			int& lognumber			//!<Lognumber assigned
			);
		//! Add a log given an MISTRING name
		//! Does not parse like ReadGPS, only adds to LogList
		ERRVALUE AddLogName (
			const MISTRING& logname,
			int& lognumber			//!<Lognumber assigned
			);
		//! Return a log given an index
		ERRVALUE GetLog (
			UINT32 index,
			LOGRECORD& log
			);
		//! Return a log given a lognumber
		ERRVALUE GetLog (
			int lognumber, 
			LOGRECORD& log
			);
		//! Return a log given a filepath
		ERRVALUE GetLog (
			const FILEPATH& logpath, 
			LOGRECORD& log
			);
		//! Remove a log given a lognumber, removes all records associated with log
		ERRVALUE RemoveLog (int lognumber);
		//! Remove a log given a filepath, removes all records associated with log
		ERRVALUE RemoveLog (const FILEPATH& logpath);
		//! Remove All Logs
		void RemoveAllLogs ();

		//! Add a record given a POINTRECORD struct
		//! Returns -1 if record's lognumber not valid
		ERRVALUE AddPoint (POINTRECORD record);
		//! Return a record given an index
		ERRVALUE GetPoint (
			UINT32 index,
			POINTRECORD& record
			);
		//! Return a record given a DATETIME
		ERRVALUE GetPoint (
			const DATETIME& datetime,
			POINTRECORD& record
			);
		//! Remove a record given an index
		ERRVALUE RemovePoint (UINT32 index);
		//! Remove a record given a DATETIME
		ERRVALUE RemovePoint (const DATETIME& datetime);
		//! Remove all records associated with lognumber
		ERRVALUE RemovePoints (int lognumber);

		//! Get Number of LogRecords
		UINT32 GetNumLogs () const {return m_LogList.size();}
		//! Get Number of PointRecords
		UINT32 GetNumPoints () const {return m_PointList.size();}

		//! Get FILEPATH of log given log's number
		ERRVALUE GetLogPath (
			int lognumber,
			FILEPATH& logpath
			);
		//! Get log's number given FILEPATH
		ERRVALUE GetLogNum (
			const FILEPATH& logpath,
			int& lognumber
			);

		//! Get Log List
		std::vector<LOGRECORD>& GetLogList ();
		//! Get Record List
		std::vector<POINTRECORD>& GetPointList ();
		//! Get Current Offset
		double GetOffset() const {return (m_offset); };

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Used to scan lines in ReadLogText
		ERRVALUE ScanLine (
			const LOGPARMS& settings,
			char *linebuf,
			DPOINT3D& location,
			DATETIME& datetime,
			double& heading,
			double& speed
			);
		//! List of GPS Logs
		std::vector<LOGRECORD> m_LogList;
		//! List of all GPS Records
		std::vector<POINTRECORD> m_PointList;
		//! Camera Time Offset
		INT32 m_offset;
		//! Assign lognum to logs, increases by one each time a log added
		int m_lognum;
		//! priv datetime
		DATETIME m_datetime;
		#endif
	};
#endif //INC_MI32_GPSDATABASE_H
