/***************************************************************************
 *	\file mi32/httpclient.h
 *	\brief HTTPCLIENT class definitions
 *	
 *	\if NODOC
 *	$Log: httpclient.h_v $
 *	Revision 1.62  2012/05/04 15:45:47  mju
 *	Add cacheClear.
 *	Remove unused set/getCacheDirectory methods.
 *
 *	Revision 1.61  2012/05/03 20:30:28  mju
 *	Add is/setCacheDisabled.
 *
 *	Revision 1.60  2012/04/23 17:51:18  mju
 *	*** empty log message ***
 *
 *	Revision 1.59  2012/04/20 21:31:14  mju
 *	Move all private members/methods to internal priv class.
 *	Remove httpserver-only methods as don't belong in client.
 *	Add delegate support to httpclient and remove delegateclient.
 *
 *	Revision 1.58  2012/04/17 16:03:11  mju
 *	Remove unused methods.
 *
 *	Revision 1.57  2012/04/09 14:37:59  mju
 *	Add hdrs previously included by httpmsg.
 *
 *	Revision 1.56  2012/02/22 19:03:34  ldyas
 *	added extended logging.
 *
 *	Revision 1.55  2011/05/11 14:38:42  mju
 *	Order methods alphabetical.
 *
 *	Revision 1.54  2011/05/09 13:20:22  mju
 *	Remove unused setUsername/Password methods and put static method defns at beginning.
 *
 *	Revision 1.53  2011/05/02 18:52:45  ldyas
 *	changed to register delegate instead of set
 *
 *	Revision 1.52  2011/05/02 15:39:51  ldyas
 *	adjustments for static authentication support
 *
 *	Revision 1.48  2011/04/11 18:41:49  ldyas
 *	make mac and pc use same params
 *
 *	Revision 1.46  2011/04/11 17:17:26  ldyas
 *	only one postfile method currently used.
 *
 *	Revision 1.45  2011/04/11 17:08:02  ldyas
 *	removing hSession from method params.
 *
 *	Revision 1.36  2011/03/15 16:04:12  ldyas
 *	added set methods for username and password and username and password.
 *
 *	Revision 1.35  2011/01/20 17:13:01  ldyas
 *	added postFile Methods
 *
 *	Revision 1.34  2011/01/12 00:11:03  ldyas
 *	added method for download file that takes a httpmessage
 *
 *	Revision 1.30  2010/07/02 18:55:55  ldyas
 *	added Get google maps auth string and download file that takes header params
 *
 *	Revision 1.29  2010/06/11 19:35:48  ldyas
 *	default values for ignore cache and session.
 *
 *	Revision 1.25  2010/02/17 23:07:47  ldyas
 *	added method for download so that it can send a body.
 *
 *	Revision 1.24  2010/02/04 16:35:30  ldyas
 *	added get auth header
 *
 *	Revision 1.23  2009/04/22 14:01:17  mju
 *	Change setUseProxy to take host/port parms too.
 *
 *	Revision 1.20  2008/05/01 17:10:10  scowan
 *	Added a download methgod that takes an isEscaped bool.
 *
 *	Revision 1.19  2007/02/08 23:18:08  dwilliss
 *	Started adding cache control support
 *
 *	Revision 1.13  2006/06/22 13:52:51  dwilliss
 *	Added new Download methods which handle authentication and redirects
 *
 *	Revision 1.12  2006/06/21 16:16:50  dwilliss
 *	initial implementation of redirect and authentication
 *
 *	Revision 1.11  2006/06/20 21:12:02  dwilliss
 *	Made virtual methods private to conform with current style guidelines
 *	Added v_OnAuthRequest virtual method
 *	Added a derived class that works through delegates instead of overriding
 *	   virtual methods
 *
 *	Revision 1.10  2006/03/21 22:58:35  dwilliss
 *	Make the legacy send message methods non-inline.
 *	Added Doxygen comments to them saying why they're deprecated.
 *
 *	Revision 1.7  2004/03/11 21:20:03  dwilliss
 *	Changed access of socket client virtual methods
 *	cope with destorying client while its still in use
 *
 *	Revision 1.6  2004/02/16 17:24:47  dwilliss
 *	Added OnStatusBegin/End methods
 *
 *	Revision 1.2  2004/01/05 20:26:25  dwilliss
 *	\endif
 **************************************************************************/

#ifndef INC_MI32_HTTPCLIENT_H
#define INC_MI32_HTTPCLIENT_H

#ifndef INC_MI32_HTTPMSG_H
#include <mi32/httpmsg.h>
#endif

#ifndef INC_MI32_STRUTF8_H
#include <mi32/strutf8.h>
#endif

#ifndef INC_MI32_DELEGATE_H
#include <mi32/delegate.h>
#endif

#ifdef MISYSTEMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations
class MISOCKET;
class XMLDOC;
class FILEPATH;
class GENERICRW;
class STATUSCONTEXT;
#endif



//!
class CLASSLIBEXPORT HTTPCLIENT {
	public:

		typedef fastdelegate::FastDelegate<ERRVALUE (const MISTRING& host, const MISTRING& realm)> DELEGATE_ONAUTHREQUEST;
		typedef fastdelegate::FastDelegate<bool(double)> DELEGATE_ONSTATUS;
	
		class INSTANCEDATA;

		enum CACHEMODE {
			CACHEMODE_NoCache = 0,					//!< Ignore the cache completely, don't cache result
			CACHEMODE_IgnoreCache = 1,				//!< Ignore the copy in the cache (if any), but add the result to the cache for later.
			CACHEMODE_UseCacheIfCurrent = 2,		//!< If we have a copy in the cache, ask the server if it's "current" and retrieve new copy if server has a new copy.
			CACHEMODE_UseCacheIfExists = 3		//!< If there's a copy in the cache, just use it (not recommended).
			};

		enum STATE {
			STATE_Idle,
			STATE_WaitingForFirstLine,
			STATE_ReadingHeader,
			STATE_ReadingMessage
			};

		//! Clear cache contents.
		static void CacheClear ();

		//! Determine if all caching is disabled.
		static bool CacheIsDisabled ();

		//! Set whether cache is disabled.
		static void CacheSetDisabled (
			bool disable			//! Whether to disable or not
			);

		//! Get the maximum cache size in Megabytes.
		//! The cache directory is shared by all clients and is persistant.
		static INT32 GetCacheMaxSize ();

		//! Returns the status message for a given status code.
		//! These are defined by RFC-2616.  According to the standard, they
		//! MAY be localized without affecting the protocol, but they're
		//! not typically seen by the user.
		static const char* GetDefaultStatusMessage (
			int code				//!< The status code to get the message for.
			);

		//! Register common delegate to call when authorization for remote server is requested.		
		static void RegisterDelegateOnAuthRequest (
			DELEGATE_ONAUTHREQUEST delegate
			);

		//! Set the maximum cache size in Megabytes.
		//! The cache directory is shared by all clients and is persistant.
		//! Setting the max cache size below the current size will just
		//! flush the cache.
		static void SetCacheMaxSize (
			INT32 Size		//!< Max cache size in Megabytes
			);


		//! Default constructor
		HTTPCLIENT ();

		//! Create given an existing socket.
		//! If bTakeOwnership is true, then this client will "own" the
		//! socket and delete it when it's done with it.
		HTTPCLIENT (
			MISOCKET *socket,
			bool bTakeOwnership
			);

		//! Destructor
		virtual ~HTTPCLIENT ();

		//! Connect to a remote host by hostname.
		//! If given a full URL, it will extract the hostname for you.
		//! It will also parse the port number if specified as hostname:portnum
		ERRVALUE ConnectToHost (
			const char *hostname,
			int port = 80
			);

		//! Send a GET request to the server and wait for the reply.  
		//! If authentication is needed, it will call v_OnAuthRequest() to get
		//! the username and password.
		//! If it gets an HTTP redirect, it will try again at the new address.
		//! This method assumes you're allready connected to the host.
		//! According to the HTTP 1.1 protocol, the path can be either a full
		//! URI (http://hostname/abs_path) or just the "/abs_path" part.
		//!
		//! Note: if SetCacheMode() has been called to enable the cache, you
		//! will need to check if the reply message IsDirectToFile().  If so,
		//! the result is in reply.GetDestFile(), which may be a cached file.
		//! If IsDirectToFile() returns false, the result is in the message
		//! itself and can be retrieved by calling reply.GetData().  
		//! You should code for either possibility even if you think it will
		//! always come out one way or the other.  If the server says something
		//! should not be cached, it won't be.
		ERRVALUE Download (
			const char *path,		//!< Path relative to the host (i.e., without the http://hostname part)
			HTTPMESSAGE& reply	//!< message to receive the reply
			);

		//! Send a GET request to the server and wait for the reply.  
		//! If authentication is needed, it will call v_OnAuthRequest() to get
		//! the username and password.
		//! If it gets an HTTP redirect, it will try again at the new address.
		//! This method assumes you're allready connected to the host.
		//! According to the HTTP 1.1 protocol, the path can be either a full
		//! URI (http://hostname/abs_path) or just the "/abs_path" part.
		//!
		//! Note: if SetCacheMode() has been called to enable the cache, you
		//! will need to check if the reply message IsDirectToFile().  If so,
		//! the result is in reply.GetDestFile(), which may be a cached file.
		//! If IsDirectToFile() returns false, the result is in the message
		//! itself and can be retrieved by calling reply.GetData().  
		//! You should code for either possibility even if you think it will
		//! always come out one way or the other.  If the server says something
		//! should not be cached, it won't be.
		ERRVALUE Download (
			const char *path,		//!< Path relative to the host (i.e., without the http://hostname part)
			bool bIsEscaped,		//!< Pass true if uri is escaped as per RFC1738
			HTTPMESSAGE& reply	//!< message to receive the reply
			);

		//! Send a request to the server and wait for the reply.
		//! If authentication is needed, it will call v_OnAuthRequest() to get
		//! the username and password.
		//! If it gets an HTTP redirect, it will try again at the new address.
		//!
		//! See the warnings about how the cache can affect the reply in the
		//! documentation for the other Download method.
		ERRVALUE Download (
			const HTTPMESSAGE& request,	//!< Request message
			HTTPMESSAGE& reply				//!< message to receive the reply
			);

		//! Download a file from a server to a specified location.
		//! If filepath points to an existing file and the one on the server
		//! is not newer than the existing copy, nothing will be downloaded
		//! unless bIgnoreCache is true.
		//! If authentication is needed, it will call v_OnAuthRequest() to get
		//! the username and password.
		//! If it gets an HTTP redirect, it will try again at the new address.
		ERRVALUE DownloadFile (
			const char *path,
			FILEPATH& filepath,
			const MILIST<STRUTF8>& Headers,
			bool bIgnoreCache = false
			);
					
		ERRVALUE DownloadFile (
			const char *path,
			FILEPATH& filepath,
			const MILIST<STRUTF8>& Headers,
			HTTPMESSAGE& Reply,
			bool bIgnoreCache = false
			);

		ERRVALUE GetAuthentication (
			const MISTRING& host,
			const MISTRING& realm,
			MISTRING& username,
			MISTRING& password
		   );

		ERRVALUE GetAuthHeader (
			const HTTPMESSAGE& req,
			const HTTPMESSAGE& rep,
			MISTRING& header
			);

		//! Get the current CACHEMODE
		CACHEMODE GetCacheMode (
			) const;

		const MISTRING& GetHostName () const;

		//! Looks up and returns an INSTANCEDATA with the given ID.
		//! HTTPCLIENT::INSTANCEDATA is automatically attached to its client
		//! by its constructor and detached by its destructor.
		//! \param id The INSTANCEDATA's id as returned by GetID()
		//! \return A pointer to the instance data or NULL if no data with with
		//! the given id could be found.
		INSTANCEDATA* GetInstanceData (
			const char *id
			) const;

		//! Get something from the server
		//! Mainly used after a call to SendMsg() to get the reply from the HTTP server
		ERRVALUE GetMsg (
			HTTPMESSAGE& obj
			);

		ERRVALUE PostFile (
			const char *path,
			FILEPATH& responsefile,
			GENERICRW& body,
			const MILIST<STRUTF8>& headers,
			HTTPMESSAGE& Reply,
			bool bIgnoreCache = false
			);

		ERRVALUE RemoveAuthentication (
			const MISTRING& host,
			const MISTRING& realm
			);

		//! Send request and receive reply.
		ERRVALUE Send (
			const HTTPMESSAGE& Request,
			HTTPMESSAGE& Reply
			);

		//! Send a message. 
		ERRVALUE SendMsg (
			const HTTPMESSAGE& msg,				//!< The headers and body of the message
			bool bHeaderOnly = false
			);

		//! Send a message with MISTRING body
		ERRVALUE SendMsg (
			const HTTPMESSAGE& headers,
			const MISTRING& body,				//!< Message body
			bool bHeaderOnly = false
			);

		//! Send a message with STRUTF8 body
		ERRVALUE SendMsg (
			const HTTPMESSAGE& headers,
			const STRUTF8& body,					//!< Message body.
			bool bHeaderOnly = false
			);

		//! Send a message with XMLDOC body
		ERRVALUE SendMsg (
			const HTTPMESSAGE& headers,
			const XMLDOC& body,					//!< Message body
			bool bHeaderOnly = false
			);

		//! Send a message with body from GENERICRW.
		//! The body of the message is read from the GENERICRW-derived class.
		ERRVALUE SendMsg (
			const HTTPMESSAGE& headers,
			GENERICRW& body,					//!< Reader for message body
			bool bHeaderOnly = false
			);

		//! Set authentication.
		ERRVALUE SetAuthentication (
			const MISTRING& host,
			const MISTRING& realm,
			const MISTRING& username,
			const MISTRING& password
			);

		//! Set the CACHEMODE.
		//! The default cache mode is CACHEMODE_NoCache
		void SetCacheMode (
			CACHEMODE mode
			);

		//! Set delegate to call while receiving data.
		//! Delegate should return true to continue, false to cancel.
		//! Will be called several times with a value of 0.0 while reading the
		//! header.  If the message is sent "chunked" the total size isn't known
		//! until it's all read, so in that case OnStatus is called once for 
		//! each chunk with a value of -1.  This allows for a status bar to at
		//! least do something and check cancel.
		void SetDelegateOnStatus (
			DELEGATE_ONSTATUS delegate
			);

		//! Set delegate to be called before starting a message download.
		void SetDelegateOnStatusBegin (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set delegate to be called when a message download is done.
		void SetDelegateOnStatusEnd (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set the host name of the remote server.  
		//! This is needed for the GET header when requesting data.
		//! We should be able to get this from the socket, but one server could
		//! serve multiple virtual hosts, so this lets you specify which one.
		//! Note, ConnectToHost() sets this for you.
		void SetHostName (
			const char *name
			);

		//! Set the timeout interval in seconds.
		//! If not set, the default is to wait indefinately.
		void SetTimeOut (
			int seconds
			);

		//! Set whether to use proxy.
		void SetUseProxy (
			bool UseProxy,
			const char *ProxyHost = 0,
			int ProxyPort = 0
			);

	private:		
	
		#ifndef GENERATING_DOXYGEN_OUTPUT
		friend class INSTANCEDATA;	// Only this can call attach/detach instance data
		class PRIV;
		PRIV *m_pPriv;

		void AttachInstanceData (
			INSTANCEDATA *data
			);

		void DetachInstanceData (
			INSTANCEDATA *data
			);

		// Unimplemented.
		HTTPCLIENT (const HTTPCLIENT&);
		HTTPCLIENT& operator= (const HTTPCLIENT&);

		#endif // GENERATING_DOXYGEN_OUTPUT

		// Overridables

		//! Called to get username/password if requesting a page that requires it.
		virtual ERRVALUE v_OnAuthRequest (
			const MISTRING& host,
			const MISTRING& realm
			);
	
	};


//! Base-class for user-defined instance data for a client.
//! This class allows you to hang your own data off of an HTTPCLIENT.  
//! This data will automatically get deleted when the client is deleted, 
//! although you can delete it yourself before the client.
class CLASSLIBEXPORT HTTPCLIENT::INSTANCEDATA {
	public:
		INSTANCEDATA (
			HTTPCLIENT *client
			);

		virtual ~INSTANCEDATA();

		//! Get the client that this data is attached to
		HTTPCLIENT* GetClient (
			) const;

		//! Get a unique identifier string that can be used to retrieve the
		//! data from the client
		const STRUTF8& GetID (
			) const;

	private:
		HTTPCLIENT *m_client;
		STRUTF8 m_id;	
	};




#undef CLASSLIBEXPORT

#endif // INC_MI32_HTTPCLIENT_H
