/**
 * \file mi32/iniauto.h
 * \brief INI_AUTO templates
 *
 * \if NODOC
 * $Id: iniauto.h_v 1.2 2007/05/11 21:43:49 mju Exp $
 *
 * $Log: iniauto.h_v $
 * Revision 1.2  2007/05/11 21:43:49  mju
 * Turn off deprecation warnings for strdup.
 *
 * Revision 1.1  2006/09/26 12:46:00  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_INIAUTO_H
#define	INC_MI32_INIAUTO_H

#ifndef	INC_MI32_INIDEFNS_H
	#include <mi32/inidefns.h>
#endif


//! INI_AUTO is a class designed to automatically handle reading and writing of
//! data from an INI file and ensure that IniRead \ IniWrite pairs function
//! together correctly.  The constructor calls IniRead and the assignment operator calls
//! IniWrite.  The Write method allows the data to be written to the INI file
//! at times other than assignment if necessary.
//!
//! Example:
//! INI_AUTO<COLOR> m_Color;
//! In container constructor's initializer list:
//! m_Color(COLOR::RED),
template <class _DT> class INI_AUTO {

	public:

		INI_AUTO (
			const char *field,				//!< Field that the data is to be stored under
			const char *group = NULL, 		//!< Group to locate field in, uses GetAppName() if NULL
			bool copyfield = false,			//!< Keep internal copy
			bool copygroup = false			//!< Keep internal copy
			) :
			m_Group(group),
			m_Field(field),
			m_CopyGroup(copygroup),
			m_CopyField(copyfield)
			{
			if (copyfield) m_Field = strdup(field);
			if (copygroup) m_Group = strdup(group);
			IniRead(NULL, m_Group, m_Field, m_Data);
			}

		INI_AUTO (
			const _DT &defaultdata,					//!< Default value
			const char *field,				//!< Field that the data is to be stored under
			const char *group = NULL, 		//!< Group to locate field in, uses GetAppName() if NULL
			bool copyfield = false,			//!< Keep internal copy
			bool copygroup = false			//!< Keep internal copy
			) :
			m_Data(defaultdata),
			m_Group(group),
			m_Field(field),
			m_CopyGroup(copygroup),
			m_CopyField(copyfield)
			{
			if (copyfield) m_Field = strdup(field);
			if (copygroup) m_Group = strdup(group);
			IniRead(NULL, m_Group, m_Field, m_Data);
			}

		~INI_AUTO (
			) {
			if (m_CopyField) free(const_cast<char*>(m_Field));
			if (m_CopyGroup) free(const_cast<char*>(m_Group));
			}

		operator const _DT& (
			) const { return (m_Data); }

		const _DT& operator= (
			const _DT& dt
			) {
			m_Data = dt;
			Write();
			return (m_Data);
			}

		//! Alternative to assignment - allows setting of
		//! data without writing to file
		void Set (
			const _DT& dt,
			bool write = true
			) {
			m_Data = dt;
			if (write)
				Write();
			}

		//! Force write to file
		void Write (
			INIHANDLE inihandle = NULL
			) const { IniWrite(inihandle, m_Group, m_Field, m_Data); }				

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		_DT m_Data;
		const char *m_Group; 				//!< Group to locate field in, uses GetAppName() if NULL
		const char *m_Field;					//!< Field that the data is to be stored under
		bool m_CopyGroup;						//!< Internal copy (free m_Group)
		bool m_CopyField;						//!< Internal copy (free m_Field)
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//! INI_AUTO_ENUM is a class designed to automatically handle reading and writing of
//! enum data from an INI file and ensure that IniReadEnum \ IniWrite pairs function
//! together correctly.  The constructor calls IniReadEnum and the assignment operator calls
//! IniWrite.  The Write method allows the data to be written to the INI file
//! at times other than assignment.
//!
//! Example:
//!	enum COLORTYPE {...};
//! INI_AUTO_ENUM<COLORTYPE> m_ColorType;
//! In container constructor's initializer list:
//! m_ColorType(COLORTYPE_Default),
template <class _DT> class INI_AUTO_ENUM {

	public:

		INI_AUTO_ENUM (
			const _DT &defaultdata,					//!< Default value
			const char *field,				//!< Field that the data is to be stored under
			const char *group = NULL, 		//!< Group to locate field in, uses GetAppName() if NULL
			bool copyfield = false,			//!< Keep internal copy
			bool copygroup = false			//!< Keep internal copy
			) :
			m_Data(defaultdata),
			m_Group(group),
			m_Field(field),
			m_CopyGroup(copygroup),
			m_CopyField(copyfield)
			{
			#ifdef WIN32
				#pragma warning (disable:4996)
			#endif
			if (copyfield) m_Field = strdup(field);
			if (copygroup) m_Group = strdup(group);
			IniReadEnum(NULL, m_Group, m_Field, m_Data);
			#ifdef WIN32
				#pragma warning (default:4996)
			#endif
			}

		~INI_AUTO_ENUM (
			) {
			if (m_CopyField) free(const_cast<char*>(m_Field));
			if (m_CopyGroup) free(const_cast<char*>(m_Group));
			}

		operator const _DT& (
			) const { return (m_Data); }

		const _DT& operator= (
			const _DT& dt
			) {
			m_Data = dt;
			Write();
			return (m_Data);
			}

		//! Alternative to assignment - allows setting of
		//! data without writing to file
		void Set (
			const _DT& dt,
			bool write = true
			) {
			m_Data = dt;
			if (write)
				Write();
			}

		//! Force write to file
		void Write (
			INIHANDLE inihandle = NULL
			) const { IniWrite(inihandle, m_Group, m_Field, m_Data); }				

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		_DT m_Data;
		const char *m_Group; 				//!< Group to locate field in, uses GetAppName() if NULL
		const char *m_Field;					//!< Field that the data is to be stored under
		bool m_CopyGroup;						//!< Internal copy (free m_Group)
		bool m_CopyField;						//!< Internal copy (free m_Field)
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

#endif	// INC_MI32_INIAUTO_H

