/**
 * \file mi32/tiffext.h
 * \brief TIFFEXT class
 *
 * \if NODOC
 * $Id: tiffext.h_v 1.17 2011/11/08 13:41:52 mju Exp $
 *
 * $Log: tiffext.h_v $
 * Revision 1.17  2011/11/08 13:41:52  mju
 * Remove redundant hdr.
 *
 * Revision 1.16  2011/09/06 13:49:39  mju
 * Exclude macro from dox.
 *
 * Revision 1.15  2008/11/20 21:25:14  mju
 * Add photometric_cmyk.
 *
 * Revision 1.14  2008/07/01 22:18:21  bcolombe
 * Added IsBigTiff to Make
 *
 * Revision 1.13  2007/06/20 19:43:42  bcolombe
 * Added TIFFEXT_MEM class
 *
 * Revision 1.12  2007/01/19 16:09:58  mju
 * Use color_array.
 *
 * Revision 1.11  2006/05/30 17:24:04  mju
 * Add setWriteUserDefinedCRS.
 *
 * Revision 1.10  2005/12/16 16:02:51  mju
 * Add writeGeoreference variant using c-style ctrlpoint array and count.
 *
 * Revision 1.9  2005/08/29 15:45:11  scowan
 * Fixed g++ warnings.
 *
 * Revision 1.8  2005/03/31 16:57:21  fileserver!dwilliss
 * Rename one of our types to MIUNICODE because it conflicted with a Microsoft #define
 *
 * Revision 1.7  2004/07/19 16:59:15  mju
 * Use coordrefsys.
 *
 * Revision 1.6  2004/02/17 17:10:42  mju
 * Add get/set subtypeflags methods.
 *
 * Revision 1.5  2004/01/29 15:39:12  mju
 * Add ReadMetadata.
 *
 * Revision 1.4  2003/10/03 20:07:48  linux32build!build
 * Doxygen
 *
 * Revision 1.3  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.2  2003/07/24 21:46:40  mju
 * Now working.
 *
 * Revision 1.1  2003/07/11 21:40:29  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_TIFFEXT_H
#define	INC_MI32_TIFFEXT_H

#ifndef	INC_MI32_FILERAW_H
	#include <mi32/fileraw.h>
#endif

#ifndef INC_MI32_GENERICRWMEM_H
	#include <mi32/genericrwmem.h>
#endif

#ifndef	INC_MI32_COLOR_H
	#include <mi32/color.h>
#endif

#ifndef	INC_MI32_CTRLPT_H
	#include <mi32/ctrlpt.h>
#endif

#ifndef	INC_MI32_MISTRING_H
	#include <mi32/mistring.h>
#endif

#ifdef RVCSYSDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class TRANS2D_AFFINE;
namespace SPATREF {
	class COORDREFSYS;
	}
#endif

//===================================================================================================================

//! Base class for TIFF image access.
class CLASSLIBEXPORT TIFFEXT {
	public:

		enum SUBTYPEFLAGS {
			SUBTYPEFLAG_None =				0x00,
			SUBTYPEFLAG_ReducedImage =		0x01,
			SUBTYPEFLAG_Page =				0x02,
			SUBTYPEFLAG_Mask =				0x04
			};

		enum COMPRESSION {
			COMPRESSION_None =			1,			//!< dump mode
			COMPRESSION_CCITT_RLE =		2,			//!< CCITT modified Huffman RLE
			COMPRESSION_CCITT_FAX3 =	3,			//!< CCITT Group 3 fax encoding
			COMPRESSION_CCITT_FAX4 =	4,			//!< CCITT Group 4 fax encoding
			COMPRESSION_L_Z_W =			5,			//!< Lempel-Ziv  & Welch
			COMPRESSION_OldJPEG =		6,			//!< !6.0 JPEG
			COMPRESSION_JPEG_DCT =		7,			//!< %JPEG DCT compression
			COMPRESSION_NeXT =			32766,	//!< NeXT 2-bit RLE
			COMPRESSION_CCITT_RLEW =	32771,	//!< #1 w/ word alignment
			COMPRESSION_PackBits =		32773,	//!< Macintosh RLE
			COMPRESSION_ThunderScan =	32809		//!< ThunderScan RLE
			};

		enum PHOTOMETRIC {
			PHOTOMETRIC_MinIsWhite =	0,
			PHOTOMETRIC_MinIsBlack =	1,
			PHOTOMETRIC_RedGreenBlue =	2,
			PHOTOMETRIC_Palette =		3,
			PHOTOMETRIC_Mask =			4,
			PHOTOMETRIC_Separated =		5,
			PHOTOMETRIC_YCbCr =			6,
			PHOTOMETRIC_CIE_Lab =		8,
			PHOTOMETRIC_ITU_Lab =		10,
			PHOTOMETRIC_CMYK =			65530
			};

		enum PLANARCONFIG {
			PLANARCONFIG_Contiguous =	1,
			PLANARCONFIG_Separate =		2
			};

		enum RESOLUTIONUNIT {
			RESOLUTIONUNIT_Inch =			2,
			RESOLUTIONUNIT_Centimeter =	3
			};

		enum SAMPLEFORMAT {
			SAMPLEFORMAT_Unsigned =			1,
			SAMPLEFORMAT_Signed =			2,
			SAMPLEFORMAT_Float =				3,
			SAMPLEFORMAT_Undefined =		4,
			SAMPLEFORMAT_ComplexInt =		5,
			SAMPLEFORMAT_ComplexFloat =	6
			};

		#ifndef GENERATING_DOXYGEN_OUTPUT
		class IMAGEINFO;
		class IMAGEINFO_PRIV;
		#endif // GENERATING_DOXYGEN_OUTPUT

		//! Increment reference count.
		//! Reference count will be 1 after construction, only use if require reference counting.
		void AddRef (
			);

		//! Get index of 'default' image.
		int GetDefaultImageIdx (
			) const;

		//! Get information about currently opened image.
		IMAGEINFO GetImageInfo (
			) const;

		//! Get number of images in file.
		int GetNumImages (
			) const;

		//! Get index of currently opened image.
		int GetOpenImageIdx (
			) const;

		//! Make and open new image.
		//! @return New image index or error < 0.
		int MakeImage (
			const IMAGEINFO& imageinfo					//!< Settings for image to create
			);

		//! Open specified image.
		//! If file is opened for reading the first image is opened by default.
		ERRVALUE OpenImage (
			UINT16 ImageIdx
			);

		//! Read contrast table.
		//! @return TRUE if has table, FALSE if not, < 0 if error.
		int ReadContrastTable (
			SIMPLE_ARRAY<UINT8>& table					//!< Contrast table returned
			);

		//! Read georeference information.
		//! @return TRUE if has georeference, FALSE if not, < 0 if error.
		int ReadGeoreference (
			SPATREF::COORDREFSYS& CoordRefSys,		//!< Coordinate Reference System definition returned
			TRANS2D_AFFINE& TransAffine,				//!< Affine transformation returned
			SIMPLE_ARRAY<CTRLPOINT3>& CtrlPoints	//!< Control point array returned, empty if no control points
			);

		// Read metadata (properties from header).
		ERRVALUE ReadMetadata (
			MISTRING& string								//!< String to contain metadata
			);

		//! Read color palette.
		//! @return TRUE if has table, FALSE if not, < 0 if error.
		int ReadPalette (
			COLOR_ARRAY& palette			//!< Color palette returned
			);

		//! Read all or part of a line of pixels from image.
		ERRVALUE ReadPixels (
			INT32 sample,								//!< Sample index, -1 for all samples if PLANARCONFIG_Contiguous
			INT32 line,									//!< Starting line
			INT32 column,								//!< Starting column
			INT32 numpixels,							//!< Number of pixels to read
			void *buffer,								//!< Buffer to contain pixels
			bool leavepacked = false				//!< Leave buffer packed if < 8 bits per sample
			);

		//! Decrement reference count and destroy if no more references.
		//! @return 'true' if instance was deleted, false if not.
		bool Release (
			);

		//! Set to write user-defined Coordinate Reference System when writing georeference.
		//! This may improve compatibility with some GeoTIFF readers.
		void SetWriteUserDefinedCRS (
			bool WriteUserDefinedCRS = true
			);

		//! Set to write the Maximum and Minimum cell values to HALFTONEHINTS
		//! Used for linear contrast stretching
		ERRVALUE WriteContrast (
			UINT16 MaxValue,	//!< Maximum Cell Value
			UINT16 MinValue		//!< Minimum Cell Value
			);

		//! Write georeference information using TRANS2D_AFFINE.
		ERRVALUE WriteGeoreference (
			const SPATREF::COORDREFSYS& CoordRefSys,	//!< Coordinate Reference System definition
			const TRANS2D_AFFINE& TransAffine,
			bool ForceNonRotated = false
			);

		//! Write georeference information using control point array.
		ERRVALUE WriteGeoreference (
			const SPATREF::COORDREFSYS& CoordRefSys,	//!< Coordinate Reference System definition
			const CTRLPOINT3 *CtrlPoints,
			int NumCtrlPoints,
			bool ForceNonRotated = false
			);

		//! Write georeference information using control point array.
		ERRVALUE WriteGeoreference (
			const SPATREF::COORDREFSYS& CoordRefSys,	//!< Coordinate Reference System definition
			const SIMPLE_ARRAY<CTRLPOINT3>& CtrlPoints,
			bool ForceNonRotated = false
			);

		//! Write color palette.
		ERRVALUE WritePalette (
			const COLOR_ARRAY& palette
			);

		//! Write all or part of a line of pixels from image.
		ERRVALUE WritePixels (
			INT32 sample,								//!< Sample index, -1 for all samples if PLANARCONFIG_Contiguous
			INT32 line,									//!< Starting line
			INT32 column,								//!< Starting column
			INT32 numpixels,							//!< Number of pixels to read
			const void *buffer,						//!< Buffer containing pixels to write
			bool prepacked = false					//!< Is buffer pre-packed for data less than 8 bits per sample
			);

	protected:

		enum OPENMODE {
			OPENMODE_Read = 0,
			OPENMODE_Write = 1
			};

		//! Constructor.
		TIFFEXT ();

		//! Destructor.
		virtual ~TIFFEXT ();

		//! Close TIFF instance.
		ERRVALUE CloseTIFF ();

		//! Open TIFF instance.
		ERRVALUE OpenTIFF (
			GENERICRW *pGenRW,				//!< Read/write class instance, must support seeking
			const MIUNICODE *name,				//!< "Name" of file or instance
			OPENMODE openmode,					//!< OPENMODE_Read or OPENMODE_Write,
			bool IsBigTiff = false
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;

		TIFFEXT (const TIFFEXT&);
		TIFFEXT& operator= (const TIFFEXT&);
		friend class TIFFEXT::PRIV;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(TIFFEXT::SUBTYPEFLAGS)
#endif


//===================================================================================================================

//! Container for information about specific TIFF image.
class CLASSLIBEXPORT TIFFEXT::IMAGEINFO {
	public:

		//! Constructor for setting up new info.
		IMAGEINFO (
			INT32 ImageLength,						//!< Image length (number of lines)
			INT32 ImageWidth,							//!< Image width (number of columns)
			UINT16 SamplesPerPixel,					//!< Number of samples (components) per pixel
			UINT16 BitsPerSample,					//!< Number of bits per sample, same for all samples
			SAMPLEFORMAT SampleFormat = SAMPLEFORMAT_Unsigned	//!< Sample numeric format
			);

		#ifndef GENERATING_DOXYGEN_OUTPUT
		IMAGEINFO (IMAGEINFO_PRIV*);
		#endif // GENERATING_DOXYGEN_OUTPUT

		//! Copy constructor.
		IMAGEINFO (
			const IMAGEINFO& rhs
			);

		//! Destructor.
		~IMAGEINFO (
			);

		//! Assignment operator.
		IMAGEINFO& operator= (
			const IMAGEINFO& rhs
			);

		//! Get number of bits per sample.
		//! Although the TIFF spec allows a different number of bits for each sample, the libtiff
		//! implementation does not, so at present all samples must have the same number of bits.
		UINT16 GetBitsPerSample (
			) const;

		//! Get compression method.
		COMPRESSION GetCompression (
			) const;

		//! Get image description.
		MISTRING GetDescription (
			) const;

		//! Get image length (number of lines).
		INT32 GetImageLength (
			) const;

		//! Get image width (number of columns).
		INT32 GetImageWidth (
			) const;

		//! Get photometric intepretation.
		PHOTOMETRIC GetPhotometric (
			) const;

		//! Get planar configuration.
		//! @return PLANARCONFIG_Contiguous (pixel-interleaved) or PLANARCONFIG_Separate (band sequential)
		PLANARCONFIG GetPlanarConfig (
			) const;

		//! Get sample numeric format.
		//! Although the TIFF spec allows a different numeric format for each sample, the libtiff
		//! implementation does not, so at present all samples must have the same numeric format.
		SAMPLEFORMAT GetSampleFormat (
			) const;

		//! Get 'name' of sample.
		//! @return String, guaranteed not to be empty for valid sample index.
		MISTRING GetSampleName (
			INT16 sample								//!< Sample index
			) const;

		//! Get number of samples per pixel.
		UINT16 GetSamplesPerPixel (
			) const;

		//! Get segment (tile or strip) length (number of lines).
		INT32 GetSegmentLength (
			) const;

		//! Get segment (tile or strip) width (number of columns).
		INT32 GetSegmentWidth (
			) const;

		//! Get subtype flags.
		SUBTYPEFLAGS GetSubTypeFlags (
			) const;

		//! Determine if image is 'tiled'.
		bool IsTiled (
			) const;

		//! Set compression type.
		void SetCompression (
			COMPRESSION compression
			);

		//! Set image description.
		void SetDescription (
			const MISTRING& desc
			);

		//! Set photometric interpretation.
		void SetPhotometric (
			PHOTOMETRIC photometric
			);

		//! Set planar configuration.
		void SetPlanarConfig (
			PLANARCONFIG PlanarConfig
			);

		//! Set resolution.
		void SetResolution (
			double xres,
			double yres,
			RESOLUTIONUNIT resunit
			);

		//! Set segment size.
		//! Actual segment size may be adjusted based on format requirements.
		//! Adjusted size may be retrieved via GetSegmentHeight/Width.
		void SetSegmentSize (
			INT32 SegmentLength,
			INT32 SegmentWidth
			);

		//! Set image subtype flags.
		void SetSubTypeFlags (
			SUBTYPEFLAGS SubTypeFlags
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		IMAGEINFO_PRIV *m_pPriv;
		void GetExclusive();
		friend class TIFFEXT;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

class CLASSLIBEXPORT TIFFEXT_FILE : public TIFFEXT {
	public:

		//! Constructor.
		TIFFEXT_FILE ();

		//! Destructor.
		~TIFFEXT_FILE ();

		//! Close file.
		ERRVALUE Close ();

		FILEPATH GetPath () const;

		//! Make new TIFF file.
		ERRVALUE Make (
			const FILEPATH& filepath,
			bool IsBigTiff = false
			);

		//! Open existing TIFF file for reading.
		ERRVALUE Open (
			const FILEPATH& filepath
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		FILE_RAW m_file;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//! Class for handling TIFF memory buffers.  
class CLASSLIBEXPORT TIFFEXT_MEM : public TIFFEXT {
	public:

		//! Constructor.
		TIFFEXT_MEM ();

		//! Destructor.
		~TIFFEXT_MEM ();

		//! Close file.
		ERRVALUE Close ();

		//! Get the internal buffer.
		const UINT8* GetBuffer (
			) const;

		//! Get the size of the internal buffer.  Warning - this is the internal buffer size, 
		//! which is not necessarily the size of the JPEG-compressed data.
		INT64 GetBufferSize (
			) const;

		//! Get the internal buffer and transfer ownership to caller (i.e. you must free it).
		UINT8* Detach (
			);

		//! Create a JPEG buffer.  The caller does NOT retain ownership of the buffer passed.
		//! If the buffer is not sufficiently large to store the entire image,
		//! it will be resized.
		ERRVALUE Make (
			UINT8*& buf,
			UINT32 bufsize
			);

		//! Open an existing JPEG buffer.
		ERRVALUE Open (
			UINT8*& buf,
			UINT32 bufsize
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		GENERICRW_MEMORY m_mem;
		#endif
	};

//===================================================================================================================

#undef	CLASSLIBEXPORT

#endif	// INC_MI32_TIFFEXT_H
