/**
 * \file rvc/autolabl.h
 * \brief interface for the RVC::AUTOLABEL classes.
 *
 * \if NODOC
 * $Id: autolabl.h_v 1.14 2012/08/27 20:53:57 bcolombe Exp $
 *
 * $Log: autolabl.h_v $
 * Revision 1.14  2012/08/27 20:53:57  bcolombe
 * Add option to override computed angle of baseline
 *
 * Revision 1.13  2011/12/06 22:08:14  mju
 * Remove redundant hdrs.
 *
 * Revision 1.12  2007/12/18 21:44:12  mju
 * Add computeStringExtents.
 * Add ybase parm to addPolyLabel.
 *
 * Revision 1.2  2002/02/04 20:58:35  scowan
 * Added conglomerate line label generator.
 *
 * Revision 1.1  2002/01/11 22:49:26  scowan
 * Initial revision
 * \endif
**/

#ifndef INC_RVC_AUTOLABL_H
#define INC_RVC_AUTOLABL_H

#ifndef INC_RVC_VECTOR_H
	#include <rvc/vector.h>
#endif

#ifndef	INC_MI32_TRANS2DAFFINE_H
	#include <mi32/trans2daffine.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
namespace {
	struct SPANINFO;
	class SPANDEVICE;
	}
namespace MGD {
	struct GLYPHINFO;
	}
#endif


namespace RVC {

class AUTOLABEL {
	public:
	
		enum LINECURVE {
			LINECURVE_Straight,
			LINECURVE_Exact,
			LINECURVE_Spline
			};
			
		enum LABELORIENT {
			LABELORIENT_TopLtoR,
			LABELORIENT_TopRtoL
			};
			
		enum POSNMODE_POLYGON {
			POSNMODE_POLYGON_PolygonInsideIfFitElseOutsideWithLeader = 0,
			POSNMODE_POLYGON_PolygonInsideAlways = 1,
			POSNMODE_POLYGON_PolygonInsideIfFitElseNone = 2,
			POSNMODE_POLYGON_PolygonInsideIfFitElseOutsideNoLeader = 3
			};

		#ifndef GENERATING_DOXYGEN_OUTPUT	
		class FILTER;
		#endif // GENERATING_DOXYGEN_OUTPUT

		//! Default ctor	
		AUTOLABEL (
			);
	
		//! Destructor
		~AUTOLABEL (
			);

		//! Generate labels along a conglomerate line.  
		//! Which lines are "joined" is determined by the 'FILTER' interface class.  Lines are
		//! "joined by the label string and height returned from the filter being equal.
		ERRVALUE GenerateConglomerateLineLabel (
			INT32 StartLineNum,								//!< Initial line number to start with
			const MISTRING& string,							//!< Label string
			double height,										//!< Label height
			const RVC::OBJECTNAME& StyleName,			//!< Name of the text style entry to assign to the label
			const TEXTSTYLE& TextStyle,					//!< Style information for the label
			MILIST<RVC::VECTOR::LABEL>& LabelList,		//!< List of labels constructed (RETURNED)
			FILTER& LineFilter								//!< Filter for determining which lines to conglomerate
			);
	
		//! Auto-generate a label based on a vector element
		ERRVALUE GenerateLabel (
			ELEMTYPE ElemType,								//!< Vector element type to generate labels for
			INT32 ElemNum,										//!< Element number to generate the label for
			const MISTRING& string,							//!< Label string
			double height,										//!< Label height
			const RVC::OBJECTNAME& StyleName,			//!< Name of the text style entry to assign to the label
			const TEXTSTYLE& TextStyle,					//!< Style information for the label
			MILIST<RVC::VECTOR::LABEL>& LabelList,		//!< List of labels constructed (RETURNED)
			bool bOverrideAngle = false,					//!< Override computed baseline angle
			double OverrideAngle = 0.0							//!< Angle to use
			);
	
		//! Auto-generate a label based on a vector line element
		ERRVALUE GenerateLineLabel (
			INT32 LineNum,										//!< Line number to generate the label for
			const MISTRING& string,							//!< Label string
			double height,										//!< Label height
			const RVC::OBJECTNAME& StyleName,			//!< Name of the text style entry to assign to the label
			const TEXTSTYLE& TextStyle,					//!< Style information for the label
			MILIST<RVC::VECTOR::LABEL>& LabelList,		//!< List of labels constructed (RETURNED)
			bool bOverrideAngle = false,					//!< Override computed baseline angle
			double OverrideAngle = 0.0							//!< Angle to use
			);
	
		//! Auto-generate a label based on a vector point element
		ERRVALUE GeneratePointLabel (
			INT32 PointNum,									//!< Point number to generate the label for
			const MISTRING& string,							//!< Label string
			double height,										//!< Label height
			const RVC::OBJECTNAME& StyleName,			//!< Name of the text style entry to assign to the label
			const TEXTSTYLE& TextStyle,					//!< Style information for the label
			MILIST<RVC::VECTOR::LABEL>& LabelList,		//!< List of labels constructed (RETURNED)
			bool bOverrideAngle = false,					//!< Override computed baseline angle
			double OverrideAngle = 0.0							//!< Angle to use
			);
	
		//! Auto-generate a label based on a vector polygon element
		ERRVALUE GeneratePolygonLabel (
			INT32 PolyNum,										//!< Polygon number to generate the label for
			const MISTRING& string,							//!< Label string
			double height,										//!< Label height
			const RVC::OBJECTNAME& StyleName,			//!< Name of the text style entry to assign to the label
			const TEXTSTYLE& TextStyle,					//!< Style information for the label
			MILIST<RVC::VECTOR::LABEL>& LabelList,		//!< List of labels constructed (RETURNED)
			bool bOverrideAngle = false,					//!< Override computed baseline angle
			double OverrideAngle = 0.0							//!< Angle to use
			);

		//! Get the current polygon label positioning setting
		//! @return The current POSNMODE_POLYGON
		POSNMODE_POLYGON GetPosnModePolygon (
			) const { return (m_PosnModePoly); }
			
		//! Get the spacing between labels for large polygons and long lines	
		//! @return Label spacing in vector units
		double GetLabelSpace (
			) const { return (m_LabelSpace); }
			
		//! Get the label baseline curvature setting for line label generation
		//! @return Current line label baseline curvature setting.
		LINECURVE GetLineCurve (
			) const { return (m_LineCurve); }
			
		//! Get the label orientation setting for line labels
		//! @return Line label orientation setting
		LABELORIENT GetOrientation (
			) const { return (m_Orientation); }
			
		//! Initialize auto label polygon 
		ERRVALUE Initialize (
			const RVC::OBJITEM& ObjItem					//!< Vector object to auto-generate labels on
			);

		//! Has the auto-generate label class instance been initialized?
		bool IsInitialized (
			) const { return (m_VectorObj.IsOpen()); }

		//! Generate straight line labels orthogonal to the line?
		bool IsOrthogonalLabels (
			) const { return (m_MakeOrthogonal); }
			
		//! Set the current polygon label positioning setting
		//! @return The current POSNMODE_POLYGON 
		void SetPosnModePolygon (
			POSNMODE_POLYGON PosnModePoly
			) { m_PosnModePoly = PosnModePoly; }

		//! Set the minimum line length necessary to generate a label
		//! The length is in vector units
		void SetMinLineLength (
			double MinLineLength
			) { m_MinLineLength = MinLineLength; }
			
		//! Set the spacing between labels for large polygons and long lines
		//! Set to '0.0' to disable label spacing (default)
		void SetLabelSpace (
			double LabelSpace
			) { m_LabelSpace = LabelSpace; }

		//! Set to generate line label baselines from the section of the line
		void SetLineCurve (
			LINECURVE LineCurve
			) { m_LineCurve = LineCurve; }
			
		//! Set label orientation
		void SetOrientation (
			LABELORIENT Orientation
			) { m_Orientation = Orientation; }

		//! Set to generate line label baselines orthogonal to the line
		void SetOrthogonalLabels (
			bool value
			) { m_MakeOrthogonal = value; }

		//! Stop the auto-generate label toolkit			
		void Stop (
			);
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		
		VECTOR m_VectorObj;
		SPANDEVICE* m_SpanDevice;
		SIMPLE_ARRAY<LPOLYGON> m_GCPoly;
		SIMPLE_ARRAY<LPOINT2D> m_GCLine;
		SIMPLE_ARRAY<INT32> m_NumShellLines;	
		SIMPLE_ARRAY<INT32> m_ShellList;	
		TRANS2D_AFFINE m_VectorToGC;
		POLYLINE m_PolyLine;					//!< Polygon work buffer
		POLYLINE m_IsleLine;					//!< Isle work buffer
		POLYLINE m_TempLine;					//!< Line work buffer
		BITSET m_CheckedSet;
		LRECT2D m_LabelExtents;
		INT32 m_LabelHeight;
		bool m_MakeOrthogonal;
		bool m_AdjustLabelHeight;
		double m_LabelSpace;
		double m_MinLineLength;
		LINECURVE m_LineCurve;
		LABELORIENT m_Orientation;
		POSNMODE_POLYGON m_PosnModePoly;
		
		ERRVALUE AddPolygonLabelToList (MILIST<RVC::VECTOR::LABEL>& LabelList, INT32 PolyNum, const MISTRING& string, double height, double ybase, const RVC::OBJECTNAME& StyleName, const DRECT2D& LabelRect, bool GenerateLeaderLine = false, bool bOverrideAngle = false, double OverrideAngle = 0.0);
		void AdjustBaseline (POLYLINE& Baseline, bool bOverrideAngle = false, double OverrideAngle = 0.0);
		ERRVALUE ComputeStringExtents (const TEXTSTYLE& TextStyle, const DPOINT2D& point, double height, const MISTRING& string, DRECT2D& extents);
//		ERRVALUE GetStringMetrics (const TEXTSTYLE& TextStyle, const DPOINT2D& point, double height, const MISTRING& string, MGD::GLYPHINFO& StringMetrics);
		ERRVALUE SetupPolygon (INT32 PolyNum);
		void AdjustAffine ();
			
		AUTOLABEL (const AUTOLABEL&);
		AUTOLABEL& operator=(const AUTOLABEL&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};
	

//! Interface class to filter vector elements to see what label goes with what element
//! Currently used for line label conglomeration
class AUTOLABEL::FILTER {
	public:
		virtual bool Filter (
			ELEMTYPE ElemType,
			INT32 ElemNum,
			const MISTRING& string,
			double height
			) = 0;
	};

}		//! End of RVC namespace

#endif
