/**
 * \file rvc/cadelement.h
 * \brief RVC::CADELEMENT definitions.
 *
 * \if NODOC
 * $Id: cadelement.h_v 1.8 2012/05/11 17:52:29 mju Exp $
 *
 * $Log: cadelement.h_v $
 * Revision 1.8  2012/05/11 17:52:29  mju
 * Add getTextLabel.
 *
 * Revision 1.7  2011/12/06 22:30:55  mju
 * Remove redundant hdr.
 *
 * Revision 1.5  2007/09/04 21:15:17  scowan
 * Updated cad element set methods with dimension parm.
 *
 * Revision 1.3  2007/03/08 17:43:13  scowan
 * Add dimension parms to element's drawElement
 *
 * Revision 1.1  2006/09/15 19:56:25  scowan
 * Initial revision
 * \endif
**/

#ifndef INC_RVC_CADELEMENT_H
#define INC_RVC_CADELEMENT_H

#ifndef	INC_RVC_GEOMETRC_H
	#include <rvc/geometrc.h>
#endif

#ifndef INC_MI32_RVCCAD_H
	#include <mi32/rvccad.h>
#endif

#ifndef INC_MI32_SCRNUNIT_H
	#include <mi32/scrnunit.h>
#endif

#ifndef INC_RVC_STYLETYPE_H
	#include <rvc/styletype.h>
#endif

#include <utility>

#ifndef GENERATING_DOXYGEN_OUTPUT	
// Forward Declarations
class LABELFRAMEPARMS;
struct LINESTYLE;
class PLANECONIC;
struct POINTSTYLE;
struct POLYSTYLE;
class SUBREGION2D;
struct TEXTSTYLE;
class TRANS2D_AFFINE;
namespace RVC {
	class STYLE;
	}
#endif

namespace RVC {

//! CAD Element Definition
//! This class is used to define and use a CAD object element
class CADELEMENT {
	public:
	
		//! Enumeration of the different CAD element types in use
		enum TYPE {
			TYPE_Insert = 0,
			TYPE_Point = 1,
			TYPE_Circle = 2,
			TYPE_Arc = 3,
			TYPE_ArcChord = 4,
			TYPE_Text = 5,
			TYPE_Line = 7,
			TYPE_RotationBox = 8,
			TYPE_Polygon = 9,
			TYPE_Mesh = 10,
			TYPE_Ellipse = 11,
			TYPE_EllipticalArc = 12,
			TYPE_ArcWedge = 14,
			TYPE_EllipticalArcWedge = 15,
			TYPE_EllipticalArcChord = 16,
			TYPE_MultiLine = 17,
			TYPE_MultiPoint = 18,
			TYPE_MultiPolygon = 19,
			TYPE_LAST
			};

		#ifndef GENERATING_DOXYGEN_OUTPUT	
		class BOX;
		class DRAW;
		class INSERT;
		class MESH;
		#endif

		//! Default Constructor		
		CADELEMENT (
			);

		//! Copy Constructor		
		CADELEMENT (
			const CADELEMENT& rhs
			);

		//! Destructor		
		~CADELEMENT (
			);
			
		//! Assignment Operator
		CADELEMENT& operator= (
			const CADELEMENT& rhs
			);

		//! Clear the element
		void Clear (
			);
			
		//! Compute the closest distance from an element to the point given
		//! NOTE: Does not compute distance from a TYPE_Insert element
		double ComputeDistanceFrom (
			const DPOINT2D& point
			) const;

		//! Compute the extents of this element			
		//! NOTE: Does not compute extents from a TYPE_Insert element, use ComputeInsertExtents()
		//! @return 'True' if the extents can be calculated, 'false' if not
		bool ComputeExtents (
			DRECT3D& Extents,
			const RVC::STYLE* StyleObj = 0
			) const;

		//! Compute the border polygon (rotated box) of the inserted CAD block defined by this element
		//! @return 'True' if the element is a TYPE_Insert, 'false' if not
		bool ComputeInsertBorder (
			const CAD& CadObj,
			POLYLINE& Border
			) const;

		//! Compute the extents of the inserted CAD block defined by this element
		//! @return 'True' if the element is a TYPE_Insert, 'false' if not
		bool ComputeInsertExtents (
			const CAD& CadObj,
			DRECT3D& Extents
			) const;

		//! Convert point coordinates using affine transformation		
		void ConvertForward (
			const TRANS2D_AFFINE& taf
			);
		
		//! Convert point coordinates using generic transformation		
		ERRVALUE ConvertForward (
			const TRANS2D_MAPGEN& tmg
			);

		//! Convert point coordinates using generic transformation with densification
		ERRVALUE ConvertForwardDense (
			const TRANS2D_MAPGEN& tmg
			);

		//! Convert point coordinates using 3D perspective transformation		
		void ConvertForward (
			const TRANS3D& t3d
			);
		
		//! Convert point coordinates using affine transformation		
		void ConvertInverse (
			const TRANS2D_AFFINE& taf
			);
		
		//! Convert point coordinates using generic transformation		
		ERRVALUE ConvertInverse (
			const TRANS2D_MAPGEN& tmg
			);

		//! Convert point coordinates using generic transformation with densification
		ERRVALUE ConvertInverseDense (
			const TRANS2D_MAPGEN& tmg
			);

		//! Convert point coordinates using 3D perspective transformation		
		void ConvertInverse (
			const TRANS3D& t3d
			);

		//! 'Draw' the element using the "DRAW" interface class passed in
		//! This method takes the individual elements and does a transformation to a well known construct to simplify
		//! handling of multiple data types.  See RVC::CADELEMENT::DRAW class for the primitives that the CADELEMENT class 
		//! is transformed to.
		//! NOTE: Does not 'draw' from an TYPE_Insert element
		ERRVALUE DrawElement (
			DRAW& Draw,
			const TRANS2D_AFFINE& BlockToCAD,		//!< Affine transformation from inserted block to CAD object coordinates
			double MinChordDist = 0.0,					//!< Default is to use %5 of the Radius length
			DIMENSION dim = DIMENSION_2D
			);

		//! Get the approximate element type
		//! @return ELEMTYPE_Point, ELEMTYPE_Line, ELEMTYPE_Polygon, ELEMTYPE_Label, or ELEMTYPE_Empty if cannot be represented
		ELEMTYPE GetApproximateElementType (
			) const;
			
		//! Get the TYPE_RotationBox from this
		//! @return 'True' if the element is a TYPE_RotationBox, 'false' if not
		bool GetBox (
			BOX& Box
			) const;
			
		//! Get the database index for this element
		//! @return '-1' if no database element index exists for this element, >=0 for valid database element index
		INT32 GetDatabaseIndex (
			) const;
		
		//! Get the CADELEMENT::TYPE designation for this
		//! @return CADELEMENT::TYPE
		TYPE GetElementType (
			) const;
		
		//! Get the TEXTID for the CADELEMENT::TYPE specified.
		static TEXTID GetElementTypeTextID (
			TYPE CadType
			);

		//! Get the TEXTSTYLE for this type
		//! @return 'true' if the method is able to set the element style based on its element type, 'false' if not
		bool GetElementStyle (
			TEXTSTYLE& textstyle
			) const;
	
		//! Get the POINTSTYLE for this type
		//! @return 'true' if the method is able to set the element style based on its element type, 'false' if not
		bool GetElementStyle (
			POINTSTYLE& pointstyle
			) const;
	
		//! Get the LINESTYLE for this type
		//! @return 'true' if the method is able to set the element style based on its element type, 'false' if not
		bool GetElementStyle (
			LINESTYLE& linestyle
			) const;
	
		//! Get the POLYSTYLE for this type
		//! @return 'true' if the method is able to set the element style based on its element type, 'false' if not
		bool GetElementStyle (
			POLYSTYLE& polystyle
			) const;
			
		//! Get the GML element string based on 'this'.
		//! @return 'True' if the element can be converted, 'false' if not and the 'GMLElement' will be empty
		bool GetGMLElement (
			MISTRING& GMLElement,
			const MILIST<std::pair<MISTRING, MISTRING> >& AttribList			//!< List of attributes to add to the element definition (first -> name, second -> value)
			);
			
		//! Get the TYPE_Insert from this
		//! @return 'True' if the element is a TYPE_Insert, 'false' if not
		bool GetInsert (
			INSERT& Insert
			) const;
		
		//! Get the Text Label frame parms information from this
		//! @return 'True' if the element is a TYPE_Text type, 'false' if not
		bool GetLabelFrameParms (
			LABELFRAMEPARMS& FrameParms
			) const;
			
		//! Get the line primitives from this
		//! @return 'True' if the element is made up of a list of POLYLINE's, 'false' if not
		bool GetLines (
			POLYLINELIST& PolyLineList
			) const;
			
		//! Get the TYPE_Mesh from this
		//! @return 'True' if the element is a TYPE_Mesh, 'false' if not
		bool GetMesh (
			MESH& Mesh
			) const;
		
		//! Get the PLANECONIC representation from this
		//! @return 'True' if the element is a PLANECONIC type, 'false' if not
		bool GetPlaneConic (
			PLANECONIC& Conic
			) const;
			
		//! Get the point primitives from this
		//! @return 'True' if the element is made up of one or more points, 'false' if not
		bool GetPoints (
			POLYLINE& PointArray
			) const;
			
		//! Get the REGION2D representation from this
		//! @return 'True' if the element is a REGION2D type, 'false' if not
		bool GetRegion (
			REGION2D& Region,
			bool Validate = false
			) const;
			
		//! Get the SUBREGION2D representation from this
		//! @return 'True' if the element is a SUBREGION2D type, 'false' if not
		bool GetSubRegion (
			SUBREGION2D& SubRegion,
			bool Validate = false
			) const;
			
		//! Get the Text element information from this
		//! @return 'True' if the element is a TYPE_Text type, 'false' if not
		bool GetTextElement (
			POLYLINELIST& PolyLineList,		//!< First line is the baseline, the rest are leader lines
			MISTRING& text,
			TEXTSTYLE& style,
			LABELFRAMEPARMS* FrameParms = 0
			) const;

		//! Get text element as GEOMETRIC::LABEL.
		bool GetTextLabel (
			GEOMETRIC::LABEL& label				//!< Label returned
			) const;
			
		//! Get the units of width	
		SCREENUNIT GetWidthUnits () const;

		//! Get the Z value assiociated with this element
		double GetZValue () const;
			
		//! Does the label follow the baseline?
		//! @return 'True' if label follows baseline
		bool IsBaselineFit () const;

		//! Does the label follow the splined baseline?
		//! @return 'True' if label follows the splined baseline
		bool IsBaselineSplineFit () const;
	
		//! Clip vector features under the label when rendering
		//! @return 'True' if features under the label needs to be clipped
		bool IsClipUnderSet () const;
			
		//! Is the element an insertion element?
		//! @return 'True' if the element is a TYPE_Insert, 'false' if not
		bool IsInsertion () const;
	
		//! Can the element be represented by open lines?
		//! @return 'True' if the element can be represented by open lines, 'false' if not
		bool IsLine () const;
	
		//! Can the element be represented by a PLANECONIC
		//! @return 'True' if the element can be represented by a PLANECONIC, 'false' if not
		bool IsPlaneConic () const;
	
		//! Can the element be represented by one or more points?
		//! @return 'True' if the element can be represented by one or more points, 'false' if not
		bool IsPoint () const;

		//! Is the point inside the element?	
		//! @return 'True' if the element is a polygon (IsPolygon() == true) and the point is inside it, 'false' if not
		bool IsPointInside (
			const DPOINT2D& pt
			) const;
	
		//! Can the element be represented by closed lines?
		//! @return 'True' if the element can be represented by closed lines, 'false' if not
		bool IsPolygon (
			) const;
	
		//! Is the element a text element?
		//! @return 'True' if the element is a TYPE_Text, 'false' if not
		bool IsText (
			) const;

		//! Is the element valid?  Does 'this' represent a CAD element
		//! @return 'True' if 'this' is a valid CAD element, 'false' if not
		bool IsValid (
			) const;
	
		//! Set the TYPE_RotationBox from 'Box'
		void SetBox (
			const BOX& Box
			);
		
		//! Set the label to follow the baseline
		void SetBaselineFit (
			bool value
			);
			
		//! Set the label to follow the splined baseline
		void SetBaselineSplineFit (
			bool value
			);
			
		//! Set the label clip under flag
		void SetClipUnder (
			bool value
			);
			
		//! Set the database index for this element: '-1' if no database element index exists for this element, >=0 for valid database element index
		void SetDatabaseIndex (
			INT32 index
			);

		//! Set 'this' based on the GML element which MUST derive from gml:AbstractGeometryType
		//! @return 'True' if the element can be converted, 'false' if not
		bool SetElementFromGML (
			const MISTRING& GMLElement,
			MILIST<std::pair<MISTRING, MISTRING> >& AttribList			//!< List of attributes not used in 'this' (first -> name, second -> value)
			);
		
		//! Set the TEXTSTYLE for this type
		void SetElementStyle (
			const TEXTSTYLE& textstyle
			);
	
		//! Set the POINTSTYLE for this type
		void SetElementStyle (
			const POINTSTYLE& pointstyle
			);
	
		//! Set the LINESTYLE for this type
		void SetElementStyle (
			const LINESTYLE& linestyle
			);
	
		//! Set the POLYSTYLE for this type
		void SetElementStyle (
			const POLYSTYLE& polystyle
			);
	
		//! Set the TYPE_Insert from 'Insert'
		void SetInsert (
			const INSERT& Insert
			);
		
		//! Set the element type and the data based on 'PolyLineList'
		void SetLines (
			const POLYLINELIST& PolyLineList,
			DIMENSION dim = DIMENSION_2D
			);
			
		//! Set the TYPE_Mesh from 'Mesh'
		void SetMesh (
			const MESH& Mesh
			);
		
		//! Set the element type and data based on 'Conic'
		void SetPlaneConic (
			const PLANECONIC& Conic
			);
	
		//! Set the element type and data based on 'point'
		void SetPoint (
			const DPOINT3D& point,
			DIMENSION dim = DIMENSION_2D
			);
			
		//! Set the element type and data based on 'PointArray'
		void SetPoints (
			const POLYLINE& PointArray,
			DIMENSION dim = DIMENSION_2D
			);
			
		//! Set the element type and data based on 'PolyLine'
		void SetPolyLine (
			const POLYLINE& PolyLine,
			DIMENSION dim = DIMENSION_2D
			);
			
		//! Set the element type and data based on 'Region'
		void SetRegion (
			const REGION2D& Region,
			DIMENSION dim = DIMENSION_2D
			);
			
		//! Set the element type and data based on 'SubRegion'
		void SetSubRegion (
			const SUBREGION2D& SubRegion,
			DIMENSION dim = DIMENSION_2D
			);
			
		//! Set the TYPE_Text from the information passed in
		void SetTextElement (
			const POLYLINELIST& PolyLineList,			//!< First line is the baseline, the rest are leader lines
			const MISTRING& text,
			const TEXTSTYLE& style,
			const LABELFRAMEPARMS* FrameParms = 0,
			DIMENSION dim = DIMENSION_2D
			);

		//! Set the units of width	
		void SetWidthUnits (
			SCREENUNIT WidthUnits
			);
			
		//! Set the Z value assiociated with this element
		void SetZValue (
			double zvalue
			);
			
		//! Test this to see if it passes the TEXTCRITERIA given using the REGION2D passed in
		//! NOTE: Cannot test a TYPE_Insert element
		//! @return 'True' if the element met the criteria, 'false' if not
		bool TestElement (
			const REGION2D& Region,
			TESTCRITERIA TestMode
			) const;
	
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT	
		CADELEMHDR m_ElemHdr;
		CADELEMOPT m_ElemOpt;
		CADELEMDATA m_ElemData;
		POLYLINE m_Points;
		MISTRING m_String;
		SIMPLE_ARRAY<INT32> m_IsleList;
		SIMPLE_ARRAY<CADSUBPOLY> m_SubPolyList;

		void ClearGeometric ();
		void ComputeStringExtents (DRECT3D& Extents, const RVC::STYLE* StyleObj = 0) const;
		void GeneratePointsFromBox (POLYLINE& PolyLine) const;
		void GetPolyLineList (POLYLINELIST& PolyLineList) const;
		void SetPolyLineList (const POLYLINELIST& PolyLineList);
		void TranslatePatterns (const SIMPLE_ARRAY<INT32> PattTrans[RVC::PATTERNTYPE_NumTypes]);

		class GMLSAX;		
		friend class GMLSAX;
		friend class CAD;
		#endif
	};
	
#ifndef GENERATING_DOXYGEN_OUTPUT	
DEFINE_ENUM_OP_INCREMENT(CADELEMENT::TYPE)
#endif


//! CAD element TYPE_RotationBox class implementation
class CADELEMENT::BOX {
	public:
		//! Default Constructor		
		BOX () : m_Rotation(0.0) {}

		//! Copy Constructor		
		BOX (const BOX& rhs) : m_rect(rhs.m_rect), m_Rotation(rhs.m_Rotation) {}

		//! Assignment Operator
		BOX& operator= (
			const BOX& rhs
			) { 
			if (this != &rhs) {
				m_rect = rhs.m_rect; 
				m_Rotation = rhs.m_Rotation;
				} 
			return (*this); 
			}

		//! Clear the box
		void Clear (
			) {
			m_rect.SetInvalid();
			m_Rotation = 0.0;
			return;
			}

		// Generate a POLYLINE describing the box			
		void GeneratePoints (
			POLYLINE& PolyLine,
			DIMENSION dim
			) const;
		
		//! Get the box rectangle, unrotated
		//! @return The box rectangle
		const DRECT3D& GetBoxExtents (
			) const { return (m_rect); }
	
		//! Get the box rotation value in radians
		//! @return The box rotation value in radians
		double GetRotation (
			) const { return (m_Rotation); }
	
		//! Set the box rectangle, unrotated
		void SetBoxExtents (
			const DRECT3D& rect
			) { m_rect = rect; }
	
		//! Set the box rotation value in radians
		void SetRotation (
			double Rotation
			) { m_Rotation = Rotation; }
	
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT	
		DRECT3D m_rect;
		double m_Rotation;
		#endif
	};


//! CADELEMENT Interface class to render and/or transform CAD Elements into common types
class CADELEMENT::DRAW {
	public:
		DRAW () {}
		virtual ~DRAW () {}

		//! First chance to render an element without any transformations to one of methods below.
		//! @return >0 if this method used the element and do not call methods below, ==0 if call the methods below, <0 error
		int DrawElement (
			const CADELEMENT& Element,			//!< Original CADELEMENT
			const TRANS2D_AFFINE& BtoC				//!< Block to CAD transformation, methods below will do the transform before sending the converted element
			) { return (v_DrawElement(Element, BtoC)); }

		//! Transform CAD element to a set of one or more lines, only if the element can be represented as lines and not polygons
		ERRVALUE DrawLines (
			const CADELEMENT& Element,			//!< Original CADELEMENT
			const POLYLINELIST& LineList			//!< CADELEMENT transformed to lines
			) { return (v_DrawLines(Element, LineList)); }

		//! Transform CAD element to a set of one or more singular points, only if the element can be represented as singular points
		ERRVALUE DrawPoints (
			const CADELEMENT& Element,			//!< Original CADELEMENT
			const POLYLINE& Points					//!< CADELEMENT transformed to a POLYLINE of points
			) { return (v_DrawPoints(Element, Points)); }

		//! Called if the CAD element is a text label element
		ERRVALUE DrawText (
			const CADELEMENT& Element,			//!< Original CADELEMENT
			const MISTRING& string,					//!< Text string
			const POLYLINELIST& LineList			//!< First line is the baseline, all others after that are leader lines
			) { return (v_DrawText(Element, string, LineList)); }

		//! Transform CAD element to a SUBREGION2D, only if the element can be represented as a SUBREGION2D
		//! Multiple calls to this method will be done if one SUBREGION2D is not enough (CADELEMENT::TYPE_MultiPolygon)
		ERRVALUE DrawSubRegion (
			const CADELEMENT& Element,			//!< Original CADELEMENT
			const SUBREGION2D& SubRegion			//!< SubRegion to fill
			) { return (v_DrawSubRegion(Element, SubRegion)); }
			
	private:

	#ifdef WIN32
		#pragma warning (push)
		#pragma warning (disable:4100)	// Unreferenced formal parameter
	#endif
		virtual int v_DrawElement (const CADELEMENT& Element, const TRANS2D_AFFINE& BtoC) { return (0); }
		virtual ERRVALUE v_DrawLines (const CADELEMENT& Element, const POLYLINELIST& LineList) { return (0); }
		virtual ERRVALUE v_DrawPoints (const CADELEMENT& Element, const POLYLINE& Points) { return (0); }
		virtual ERRVALUE v_DrawText (const CADELEMENT& Element, const MISTRING& string, const POLYLINELIST& LineList) { return (0); }
		virtual ERRVALUE v_DrawSubRegion (const CADELEMENT& Element, const SUBREGION2D& SubRegion) { return (0); }
	#ifdef WIN32
		#pragma warning (pop)
	#endif
	};



//! CAD element TYPE_Insert class implementation
class CADELEMENT::INSERT {
	public:
		//! Default Constructor		
		INSERT (
			) :
			m_InsertPt(0.0,0.0,0.0),
			m_BlockNum(-1),
			m_rotate(0.0),
			m_xscale(1.0),
			m_yscale(1.0),
			m_zscale(1.0),
			m_numlins(1),
			m_numcols(1),
			m_linspace(0.0),
			m_colspace(0.0)
			{}
			
		//! Set the INSERT class back to its ctor'd state
		void Clear (
			) {
			m_InsertPt.x = m_InsertPt.y = m_InsertPt.z = m_linspace = m_colspace = m_rotate = 0.0;
			m_BlockNum = -1;
			m_xscale = m_yscale = m_zscale = 1.0;
			m_numlins = m_numcols = 1;
			return;
			}
			
		//! Get the CAD block number to insert
		//! @return The CAD block number to insert		
		INT32 GetBlockNum (
			) const { return (m_BlockNum); }
			
		//! Get the column spacing between block array insertions
		//! only useful if GetNumColumns() > 1
		//! @return Column spacing between block array insertions
		double GetColumnSpace (
			) const { return (m_colspace); }

		//! Get the block insertion point location
		//! @return Block insertion point
		const DPOINT3D& GetInsertPoint (
			) const { return (m_InsertPt); }

		//! Get the line spacing between block array insertions
		//! only useful if GetNumLines() > 1
		//! @return Line spacing between block array insertions
		double GetLineSpace (
			) const { return (m_linspace); }

		//! Get the number of columns in a multi-block array insert
		//! @return Number of columns in a multi-block array insert
		INT32 GetNumColumns (
			) const { return (m_numcols); }
			
		//! Get the number of lines in a multi-block array insert
		//! @return Number of lines in a multi-block array insert
		INT32 GetNumLines (
			) const { return (m_numlins); }
			
		//! Get the rotation angle (in radians) that the block is rotated around the insertion point
		//! @return Insertion rotation angle in radians
		double GetRotation (
			) const { return (m_rotate); }

		//! Get the x scale factor that the block is scaled around the insertion point
		//! @return Insertion X scale factor
		double GetXScale (
			) const { return (m_xscale); }

		//! Get the y scale factor that the block is scaled around the insertion point
		//! @return Insertion Y scale factor
		double GetYScale (
			) const { return (m_yscale); }

		//! Get the z scale factor that the block is scaled around the insertion point
		//! @return Insertion Z scale factor
		double GetZScale (
			) const { return (m_zscale); }

		//! Set the CAD block number to insert
		void SetBlockNum (
			INT32 BlockNum
			) { m_BlockNum = BlockNum; }
			
		//! Set the column spacing between block array insertions
		//! only useful if GetNumColumns() > 1
		void SetColumnSpace (
			double ColumnSpace
			) { m_colspace = ColumnSpace; }

		//! Set the block insertion point location
		void SetInsertPoint (
			const DPOINT3D& InsertPt
			) { m_InsertPt = InsertPt; }

		//! Set the line spacing between block array insertions
		//! only useful if GetNumLines() > 1
		void SetLineSpace (
			double LineSpace
			) { m_linspace = LineSpace; }

		//! Set the number of columns in a multi-block array insert
		void SetNumColumns (
			INT32 NumColumns
			) { m_numcols = static_cast<UINT16>(NumColumns); if (m_numcols == 0) m_numcols = 1; }
			
		//! Set the number of lines in a multi-block array insert
		void SetNumLines (
			INT32 NumLines
			) { m_numlins = static_cast<UINT16>(NumLines); if (m_numlins == 0) m_numlins = 1; }
			
		//! Set the rotation angle (in radians) that the block is rotated around the insertion point
		void SetRotation (
			double rotate
			) { m_rotate = rotate; }

		//! Set the x scale factor that the block is scaled around the insertion point
		void SetXScale (
			double xscale
			) { m_xscale = xscale; if (m_xscale == 0.0) m_xscale = 1.0; }

		//! Set the y scale factor that the block is scaled around the insertion point
		void SetYScale (
			double yscale
			) { m_yscale = yscale; if (m_yscale == 0.0) m_yscale = 1.0; }

		//! Set the z scale factor that the block is scaled around the insertion point
		void SetZScale (
			double zscale
			) { m_zscale = zscale; if (m_zscale == 0.0) m_zscale = 1.0; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT	
		DPOINT3D m_InsertPt;					//!<  Point where to insert the block 
		INT32 m_BlockNum;						//!<  Index of block to insert 
		double m_rotate;						//!<  Rotation angle 
		double m_xscale;						//!<  X Scale factor
		double m_yscale;						//!<  Y Scale factor
		double m_zscale;						//!<  Z Scale factor
		UINT16 m_numlins;						//!<  Number of lines
		UINT16 m_numcols;						//!<  Number of columns
		double m_linspace;					//!<  Spacing between lines 
		double m_colspace;					//!<  Spacing between columns 
		#endif
	};


//! CAD element TYPE_Mesh class implementation
class CADELEMENT::MESH {
	public:
		//! Default Constructor		
		MESH (
			) : 
			m_NumLineVertices(0),
			m_NumColumnVertices(0),
			m_TopBottomClosed(false),
			m_LeftRightClosed(false)
			{}
			
		//! Set the MESH class back to its ctor'd state
		void Clear (
			) {
			m_MeshPoints.Clear();			
			m_NumLineVertices = m_NumColumnVertices = 0;
			m_TopBottomClosed = m_LeftRightClosed = false;
			return;
			}
				
		//! Get the mesh point array.  
		//! The number of points is equal to GetNumColumnVertices() * GetNumLineVertices()
		//! @return A POLYLINE containing the mesh points
		const POLYLINE& GetMeshPoints (
			) const { return (m_MeshPoints); }
		
		//! Get the number of columns in the mesh
		//! @return Number of columns in the mesh
		UINT32 GetNumColumnVertices (
			) const { return (m_NumColumnVertices); }
			
		//! Get the number of lines in the mesh
		//! @return Number of lines in the mesh
		UINT32 GetNumLineVertices (
			) const { return (m_NumLineVertices); }
			
		//! Determine if the left and right sides of the mesh are to be closed
		//! @return 'True' if the left and right sides are to be closed, 'false' if not
		bool IsLeftRightClosed (
			) const { return (m_LeftRightClosed); }
	
		//! Determine if the top and bottom sides of the mesh are to be closed
		//! @return 'True' if the top and bottom sides are to be closed, 'false' if not
		bool IsTopBottomClosed (
			) const { return (m_TopBottomClosed); }

		//! Set flag to state that the left and right sides of the mesh are to be closed
		void SetLeftRightClosed (
			bool LeftRightClosed
			) { m_LeftRightClosed = LeftRightClosed; }
			
		//! Set the mesh point array.  
		//! The number of points is equal to GetNumColumnVertices() * GetNumLineVertices()
		void SetMeshPoints (
			const POLYLINE& MeshPoints
			) { m_MeshPoints = MeshPoints; }
	
		//! Set the number of columns in the mesh
		void SetNumColumnVertices (
			UINT32 NumColumnVertices
			) { m_NumColumnVertices = NumColumnVertices; }
			
		//! Set the number of lines in the mesh
		void SetNumLineVertices (
			UINT32 NumLineVertices
			) { m_NumLineVertices = NumLineVertices; }
			
		//! Set flag to state that the top and bottom sides of the mesh are to be closed
		void SetTopBottomClosed (
			bool TopBottomClosed
			) { m_TopBottomClosed = TopBottomClosed; }
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT	
		UINT32 m_NumLineVertices;						//!<  Number of X vertices for mesh 
		UINT32 m_NumColumnVertices;					//!<  Number of Y vertices for mesh 
		bool m_TopBottomClosed;
		bool m_LeftRightClosed;
		POLYLINE m_MeshPoints;
		#endif
	};

}		// End of RVC namespace

#endif 	// INC_RVC_CADELEMENT_H
