/**
 * \file rvc/stylepatternhatch.h 
 * \brief interface for the RVC::STYLEPATTERN_HATCH class.
 *
 * \if NODOC
 * $Id: stylepatternhatch.h_v 1.6 2007/04/04 17:49:27 scowan Exp $
 *
 * $Log: stylepatternhatch.h_v $
 * Revision 1.6  2007/04/04 17:49:27  scowan
 * Added virtual methods.
 *
 * Revision 1.5  2006/10/05 20:00:05  mju
 * Use point hdr.
 *
 * Revision 1.4  2006/09/21 17:16:05  scowan
 * Added more classes and methods to allow editing.
 *
 * Revision 1.3  2006/09/21 14:20:06  scowan
 * Added pattern type to the base.
 *
 * Revision 1.2  2006/09/20 16:53:56  dwilliss
 * Added ctor from raw buffer
 *
 * Revision 1.1  2006/09/18 17:54:29  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef INC_RVC_STYLEPATTERNHATCH_H
#define INC_RVC_STYLEPATTERNHATCH_H

#ifndef INC_RVC_STYLEPATTERNBASE_H
	#include <rvc/stylepatternbase.h>
#endif

#ifndef INC_MI32_POINT_H
	#include <mi32/point.h>
#endif


namespace RVC {

#ifndef GENERATING_DOXYGEN_OUTPUT
class STYLEPATTERN_LINE;
#endif // GENERATING_DOXYGEN_OUTPUT

//! Hatch pattern class.
class STYLEPATTERN_HATCH : public STYLEPATTERN_BASE {
	public:

		enum ELEMENT_TYPE {
			ELEMENT_TYPE_Simple		 = 0,
			ELEMENT_TYPE_LinePattern = 1,
			ELEMENT_TYPE_SolidFill   = 2,
			ELEMENT_TYPE_COUNT		 = 3
			};

		class ITERATOR;
		class ELEMENT;

		//! Pattern element referencing shared memory buffer.
		//! Primarily used by ITERATOR.
		class ELEMENT_SHARED {
			public:

				//! Get element type.
				//! @return Element type.
				ELEMENT_TYPE GetType (
					) const { return (static_cast<ELEMENT_TYPE>(m_pData->type)); }

				//! Get color index.
				//! @return Index into color array.
				UINT8 GetColorIndex (
					) const { return (m_pData->colornum); }

				//! Get element line pattern.
				void GetLinePattern (
					STYLEPATTERN_LINE& pattern
					) const;

				//! Get element flags.
				UINT32 GetFlags (
					) const { return (0); };

				//! Get element origin offset.
				void GetOriginOffset (
					DPOINT2D& offset
					) const { offset.x = m_pData->xoffset; offset.y = m_pData->yoffset; return; }

				//! Get spacing between repeated elements.
				//! @return spacing in 1/100 millimeter.
				double GetSpacing (
					) const { return (m_pData->spacing); }

				//! Get element angle.
				//! @return angle in radians.
				double GetAngle (
					) const { return (m_pData->angle); }

				//! Get element thickness.
				//! @return Thickness in 1/100 millimeter.
				double GetThickness (
					) const { return (m_pData->thickness); }
					
				//! Determine if element is visible.
				//! @return 'true' if visible, 'false' if not.
				bool IsVisible (
					) const { return (m_pData->isvisible != 0); }

			protected:

				//! Hatch pattern element structure.
				struct HATCHPATTELEM {
					enum TYPE {
						TYPE_Simple = 			0,	//!< Simple solid lines
						TYPE_LinePattern =	1,	//!< Line pattern
						TYPE_SolidFill =		2,	//!< Fill with solid color
						TYPE_Last =				2	//!< Last defined type
						};
						
					UINT8 type;									//!< Pattern element type: simple or line pattern
					UINT8 colornum;							//!< Color index in pattern-specific color map
					UINT8 isvisible;							//!< Flag for turning element on/off
					UINT8 space;								//!< Space
					UINT32 linedatasize;						//!< Size of the line pattern data attached
					UINT32 flags;								//!< Flags
					UINT8 spare[4];							//!< Double alignment
					DOUBLE xoffset;							//!< Origin offset
					DOUBLE yoffset;							//!< Origin offset
					DOUBLE spacing;							//!< Spacing between instances of element
					DOUBLE angle;								//!< Angle of element
					DOUBLE thickness;							//!< Thickness of element lines
					};
	
			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT

				const HATCHPATTELEM *m_pData;
				void *m_pExtra;

				//! Default constructor.
				ELEMENT_SHARED (
					): m_pData(0), m_pExtra(0) { }

				explicit ELEMENT_SHARED (const void *pElemData)
					: m_pData(static_cast<const HATCHPATTELEM*>(pElemData)), m_pExtra(static_cast<UINT8*>(const_cast<void*>(pElemData)) + sizeof(HATCHPATTELEM))
					{ }

				//! Destructor.
				~ELEMENT_SHARED (
					) { }

				friend class STYLEPATTERN_HATCH;
				friend class ELEMENT;
				friend class ITERATOR;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Pattern element.
		class ELEMENT : public ELEMENT_SHARED {
			public:

				//! Default constructor.
				ELEMENT (
					) { memset(&m_MyData,0,sizeof(m_MyData)); m_pData = &m_MyData; }

				//! Copy constructor.
				ELEMENT (
					const ELEMENT& rhs
					);

				//! Construct from ELEMENT_SHARED.
				ELEMENT (
					const ELEMENT_SHARED& rhs
					);

				//! Construct element of specified type.
				explicit ELEMENT (
					ELEMENT_TYPE type
					);

				//! Destructor.
				~ELEMENT (
					) { if (m_pExtra != 0) free(m_pExtra); }

				//! Assignment from ELEMENT.
				ELEMENT& operator= (
					const ELEMENT& rhs
					);

				//! Assignment from ELEMENT_SHARED.
				ELEMENT& operator= (
					const ELEMENT_SHARED& rhs
					);

				//! Set color index.
				void SetColorIndex (
					UINT8 ColorNum
					) { m_MyData.colornum = ColorNum; }

				//! Set element line pattern.
				void SetLinePattern (
					const STYLEPATTERN_LINE& pattern
					);

				//! Set element origin offset.
				void SetOriginOffset (
					const DPOINT2D& offset
					) { m_MyData.xoffset = offset.x; m_MyData.yoffset = offset.y; }

				//! Set spacing between repeated elements.
				void SetSpacing (
					double spacing
					) { m_MyData.spacing = spacing; }

				//! Set element angle.
				void SetAngle (
					double angle
					) { m_MyData.angle = angle; }

				//! Get element thickness.
				void SetThickness (
					double thickness
					) { m_MyData.thickness = thickness; }
					
				void SetVisible (
					bool visible
					) { m_MyData.isvisible = visible; }
					
			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				ELEMENT_SHARED::HATCHPATTELEM m_MyData;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Iterator to process pattern elements.
		class ITERATOR {
			public:

				//! Default constructor.
				ITERATOR (
					): m_pEnd(0) { }

				//! Destructor.
				~ITERATOR (
					) { }

				//! Dereference operator, returns reference to ELEMENT_SHARED.
				const ELEMENT_SHARED& operator* (
					) const { return (m_element); }

				//! Dereference operator, returns pointer to ELEMENT_SHARED.
				const ELEMENT_SHARED* operator-> (
					) const { return (&m_element); }

				//! Pre-increment operator.
				ITERATOR& operator++ ();

				//! Equality operator
				bool operator== (
					const ITERATOR& rhs
					) const { return ((m_pEnd == 0 && rhs.m_pEnd == 0) || m_element.m_pData == rhs.m_element.m_pData); }

				//! Inequality operator
				bool operator!= (
					const ITERATOR& rhs
					) const { return (!(*this == rhs)); }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT

				ELEMENT_SHARED m_element;
				const void *m_pEnd;

				ITERATOR (const void *pData, const void *pEnd)
					: m_element(pData), m_pEnd(pEnd)
					{ }

				friend class STYLEPATTERN_HATCH;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		STYLEPATTERN_HATCH (
			) : STYLEPATTERN_BASE(PATTERNTYPE_Hatch, sizeof(PATTERNHEADER_HATCH)) {}
		
		//! Constructor from raw pattern data
		STYLEPATTERN_HATCH (
			const void* buf,
			INT32 NumBytes
			) : STYLEPATTERN_BASE(PATTERNTYPE_Hatch, sizeof(PATTERNHEADER_HATCH)) {
			SetPattern(buf, NumBytes, sizeof(PATTERNHEADER_HATCH));
			}
		
		//! Copy constructor.
		STYLEPATTERN_HATCH (
			const STYLEPATTERN_HATCH& rhs
			) : STYLEPATTERN_BASE(rhs) {}

		//! Destructor.
		virtual ~STYLEPATTERN_HATCH (
			) { }

		//! Assignment.
		STYLEPATTERN_HATCH& operator= (
			const STYLEPATTERN_HATCH& rhs
			) { if (this != &rhs) { STYLEPATTERN_BASE::operator=(rhs); } return (*this); }

		//! Get iterator for beginning element.
		ITERATOR Begin (
			) const { return (ITERATOR(GetFirstElemPtr(),GetPattBuffer()+GetSize())); }

		//! Delete element at specified index.
		//! This will invalidate all ITERATORs.
		ERRVALUE DeleteElement (
			unsigned idx
			);
			
		//! Get iterator for end comparison.
		ITERATOR End (
		#ifdef WIN32
			) const { return (ITERATOR()); }
		#else
			) const { ITERATOR it; return (it); }
		#endif

		//! Get element at specified index.
		//! Note, this is less efficient than using iterator.
		ELEMENT GetElement (
			unsigned idx
			) const;

		//! Lower the specified hatch element by one
		void LowerElement (
			UINT32 Element
			);

		//! Insert element at specified index.
		//! This will invalidate all ITERATORs.
		ERRVALUE InsertElement (
			unsigned idx,
			const ELEMENT& element
			);

		//! Raise the specified hatch element by one
		void RaiseElement (
			UINT32 Element
			);

		//! Set (replace) element at specified index.
		//! If element size changes this will invalidate all ITERATORs.
		ERRVALUE SetElement (
			unsigned idx,
			const ELEMENT& element
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Hatch pattern header.
		struct PATTERNHEADER_HATCH : public STYLEPATTERN_BASE::PATTERNHEADER {
			UINT8 spare[4];							//!< Double alignment
			};

		ERRVALUE BuildOffsetIndex () const;
		INT32 GetOffset (unsigned idx) const;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


}		//! End of RVC namespace

#endif 	//!< INC_RVC_STYLEPATTERNHATCH_H
