/**
 * \file imagefiltertile.h <spatmod/imagefiltertile.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_TILE definition.
 *
 * \if NODOC
 * $Id: imagefiltertile.h_v 1.3 2011/02/03 21:25:56 mju Exp $
 *
 * $Log: imagefiltertile.h_v $
 * Revision 1.3  2011/02/03 21:25:56  mju
 * Change v_getGeoreference to use zoom.
 *
 * Revision 1.2  2009/02/09 13:51:48  mju
 * Add v_getGeoreference.
 *
 * Revision 1.1  2009/01/16 18:17:02  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERTILE_H
#define	INC_SPATMOD_IMAGEFILTERTILE_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

#ifndef	INC_SPATMOD_IMAGESECTION_H
#include <spatmod/imagesection.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Filter to tile images into single larger image.
//! First and last columns and rows may have dimensions that differ from interior tiles.
//! All interior tiles must have same dimensions.
//! Supports zooming if all tiles support compatible zooming.
class FILTER_TILE : public FILTER {
	public:

		//! Construct using specified dimensions and optional georeference.
		explicit FILTER_TILE (
			const DIMENSIONS& Dimensions,				//!< Dimensions, overall and tiling
			const GEOREFERENCE *pGeoreference = 0	//!< Optional georeference
			);

		// Destructor.
		virtual ~FILTER_TILE ();

		//! Set input stage for specific tile.
		//! All tiles must be set before calling Initialize.
		ERRVALUE SetTile (
			UINT16 TileColumn,						//!< Tile column
			UINT16 TileRow,							//!< Tile row
			STAGE *pInputStage,						//!< Pointer to stage for tile, must have correct dimensions
			bool TakeOwnership						//!< Filter takes ownership of previous stage and will delete when needed
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		class INPUTLIST;

		INPUTLIST *m_pTileListLRU;
		DIMENSIONS *m_ZoomDimensions;
		SIMPLE_ARRAY<DOUBLE> m_ZoomsAllowed;
		SECTION_ITERATOR m_TgtIterator;
		int m_ZoomIndex;
		INT32 m_MaxListSizeLRU;

		ERRVALUE BeginIteratorTileStage (INT32 TileColumn, INT32 TileRow, SECTION_ITERATOR& itTileStage) const;
		void ComputeMaxTilesPull (const SECTION_ITERATOR& TgtIterator, INT32& MaxTileColumnsPull,	INT32& MaxTileRowsPull) const;
		ERRVALUE ComputeZoomDimensions ();
		ERRVALUE RemoveLastFromLRU ();

		// BASE/STAGE/FILTER overrides.
		virtual ERRVALUE v_ComputeSampleStats (STAT_UNIVARIATE *SampleStatsArray, STATSVALIDITY StatsValidity);
		virtual ERRVALUE v_GetDimensions (DIMENSIONS& dimensions, double zoom) const;
		virtual ERRVALUE v_GetGeoreference (GEOREFERENCE& georeference, double zoom) const;
		virtual ZOOMABILITY v_GetZoomAbility (SIMPLE_ARRAY<double>& ZoomsAllowed) const;
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PullEnd ();
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);
		virtual float v_QueryIterator (const SECTION_ITERATOR& iterator) const;

		// Unimplemented
		FILTER_TILE (const FILTER_TILE&);
		FILTER_TILE& operator= (const FILTER_TILE&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERTILE_H

